import lxml.etree
import requests
import datetime

from django.conf import settings
from django.core.management.base import BaseCommand, CommandError
from django.contrib.sites.models import Site


class Command(BaseCommand):
    args = ''
    help = ''

    def handle(self, *args, **options):

        from datetime import datetime
        startTime = datetime.now()
        current_site = Site.objects.get_current()
        protocol = 'http' if not settings.USE_SSL else 'https'
        url = '%s://%s'%(protocol, current_site.domain)
        sitemap_urls = get_sitemap_urls(url)

        print "Found %s URLs to load from %s"%(len(sitemap_urls), url)
        counter = 0
        for url in sitemap_urls:
            retrieve_content(url)
            counter += 1
            print "Loaded %s of %s (%s)"%(counter, len(sitemap_urls), url)

        print "Finished loading %s sitemap URLs in %s"%(len(sitemap_urls), datetime.now() - startTime)

    
    
def get_sitemap_urls(domain):
    sitemaps = read_robots(domain)
    urls = []   
    for sitemap_url in sitemaps:
        urls += process_sitemap(sitemap_url)

    return urls


def read_robots(domain):
    
    robots_url = '%s/robots.txt'%(domain.strip("/"))
    robots_content = retrieve_content(robots_url)
    sitemaps = []


    if robots_content:
        for line in robots_content.split('\n'):
            line = line.strip()

            if line.lower().startswith('sitemap:'):
                sitemaps.append(line[len('Sitemap:'):].strip())

    return sitemaps


    
def process_sitemap(sitemap_link):    
    
    namespaces = [
        ('sm', 'http://www.sitemaps.org/schemas/sitemap/0.9'),
    ]
    
    sitemap_content = retrieve_content(sitemap_link)
    if not sitemap_content:
        return []

    urls = []
    xml = sitemap_content.encode('utf-8')
    parser = lxml.etree.XMLParser(ns_clean=True, recover=True, encoding='utf-8')
    tree = lxml.etree.fromstring(xml, parser=parser)
    
    for sitemap in tree.xpath('//sm:sitemap | //sitemap', namespaces=namespaces):
        for loc in sitemap.xpath('sm:loc | loc', namespaces=namespaces):
            child_sitemap_url = loc.text.strip()
            child_sitemap_link = self.set.get_or_create_link_object(child_sitemap_url, sitemap_link)                
            self.sitemaps.append(child_sitemap_url)
    
    for sitemap in tree.xpath('//sm:url | //url', namespaces=namespaces):
        # TODO: add last update date, rank and update frequency
        for loc in sitemap.xpath('sm:loc | loc', namespaces=namespaces):
            url = loc.text.strip()
            urls.append(url)

    return urls


def retrieve_content(url):
    r = requests.get(url)
    return r.text


from django import forms
from django.conf import settings
from django.contrib.auth import get_user_model
from django.contrib.auth.forms import UserCreationForm, AuthenticationForm, UserChangeForm, ReadOnlyPasswordHashField
from django.contrib.auth.forms import PasswordResetForm as BasePasswordResetForm
from django.contrib.auth.tokens import default_token_generator
from django.forms import extras
from django.utils.encoding import force_bytes
from django.utils.translation import ugettext_lazy as _
from django.utils.http import urlsafe_base64_encode
try:
    from django.contrib.sites.models import get_current_site
except ImportError:
    from django.contrib.sites.shortcuts import get_current_site

from ckeditorfiles.widgets import CKEditorWidget

# -----------------------------------------------------------------------------
# -- Admin/Registration Account Maintenance Forms
# -----------------------------------------------------------------------------

class UserChangeForm(forms.ModelForm):
   
    password = ReadOnlyPasswordHashField(label=_("Password"),
        help_text=_("Raw passwords are not stored, so there is no way to see "
                    "this user's password, but you can change the password "
                    "using <a href=\"password/\">this form</a>."))
    is_superuser = forms.BooleanField(required=False, label=_('User is a Super User'),
        help_text=_("User can access all areas of the CMS without having to "
        "have permissions assigned."))

    about = forms.CharField(widget=CKEditorWidget(config=settings.CKEDITOR_CONFIGS['user_about_ckeditor']), required=False)


    
    class Meta:
        model = get_user_model()

        fields = ['first_name','last_name','email','password','is_superuser']

    def clean_password(self):
        return self.initial["password"]

    def clean_email(self):
        # Since User.email is unique, this check is redundant,
        # but it sets a nicer error message than the ORM. See #13147.
        return self.cleaned_data["email"].lower()




class UserCreationForm(forms.ModelForm):
    
    
    first_name = forms.CharField(label=_("First Name"), required=True)
    last_name = forms.CharField(label=_("Last Name"), required=True)
    email = forms.RegexField(label=_("Email"), max_length=75,
        regex=r'^[\w.@+-]+$',
        help_text=_("Required. Letters, digits and @/./+/-/_ only."),
        error_messages={
            'invalid': _("This value may contain only letters, numbers and "
                         "@/./+/-/_ characters.")
        }
    )
    password1 = forms.CharField(label=_("Password"), widget=forms.PasswordInput(attrs={'placeholder': 'Password'}))
    password2 = forms.CharField(label=_("Password confirmation"),
        widget=forms.PasswordInput(attrs={'placeholder': 'Password again'}),
        help_text=_("Enter the same password as above, for verification."))


    class Meta:
        model = get_user_model()
        fields = ['first_name','last_name','email']

    def clean_email(self):
        email = self.cleaned_data["email"]
        self.cleaned_data["username"] = self.cleaned_data["email"] = email.lower()

        try:
            get_user_model().objects.get(email=email)
        except get_user_model().DoesNotExist:
            return email.lower()
        raise forms.ValidationError(settings.MESSAGES['error_duplicate_email'])

    def clean_username(self):
        return self.clean_email()

    def clean_password2(self):
        password1 = self.cleaned_data.get("password1")
        password2 = self.cleaned_data.get("password2")
        if password1 and password2 and password1 != password2:
            raise forms.ValidationError( settings.MESSAGES['error_password_mismatch'])
        return password2

    def save(self, commit=True):
        user = super(UserCreationForm, self).save(commit=False)
        
        user.set_password(self.cleaned_data["password1"])
        if commit:
            user.save()
        return user



# class PasswordResetForm(BasePasswordResetForm):
    
#     def save(self, domain_override=None,
#              subject_template_name='registration/password_reset_subject.txt',
#              email_template_name='registration/password_reset_email.html',
#              use_https=False, token_generator=default_token_generator,
#              from_email=None, request=None):
#         """
#         Generates a one-use only link for resetting password and sends to the
#         user.
#         """
#         UserModel = get_user_model()
        
#         email = self.cleaned_data["email"]
#         active_users = UserModel._default_manager.filter(email__iexact=email, is_active=True)
#         for user in active_users:
            
#             # Make sure that no email is sent to a user that actually has
#             # a password marked as unusable
#             if not user.has_usable_password():
#                 continue
            
#             if not domain_override:
#                 current_site = get_current_site(request)
#                 site = current_site
#                 domain = current_site.domain
#             else:
#                 site_name = domain = domain_override
#             c = {
#                 'email': user.email,
#                 'domain': domain,
#                 'site': site,
#                 'uid': urlsafe_base64_encode(force_bytes(user.pk)),
#                 'user': user,
#                 'token': token_generator.make_token(user),
#                 'protocol': 'https' if use_https else 'http',
#             }
#             user.send_password_reset(c)


# class CustomAuthentication(AuthenticationForm):
#     pass
    
    


# class CreateAccountForm(RegistrationFormUniqueEmail):
    
#     first_name = forms.CharField(label=_("First Name"), required=True)
#     last_name = forms.CharField(label=_("Last Name"), required=True)

    
#     next = forms.CharField(required=False)


#     def __init__(self, *args, **kwargs): 
#         super(CreateAccountForm, self).__init__(*args, **kwargs) 
        
#         self.fields.pop('username')


#     def clean_email(self):
#         """
#         Validate that the supplied email address is unique for the
#         site.
        
#         """
#         email = self.cleaned_data["email"]
#         self.cleaned_data["username"] = self.cleaned_data["email"] = email.lower()

#         if get_user_model().objects.filter(email__iexact=self.cleaned_data['email']):
#             raise forms.ValidationError(_(settings.MESSAGES['error_duplicate_email']))
#         return self.cleaned_data['email']
   


class UserGroupAdminForm(forms.ModelForm):
   
    content = forms.CharField(widget=CKEditorWidget(config=settings.CKEDITOR_CONFIGS['usergroup_content_ckeditor']), required=False)
    synopsis = forms.CharField(widget=CKEditorWidget(config=settings.CKEDITOR_CONFIGS['usergroup_synopsis_ckeditor']), required=False)



from django.contrib import admin

from carbon.atoms.admin.content import BaseVersionableAdmin, BaseVersionableTitleAdmin



class BaseRSSUrlAdmin(BaseVersionableTitleAdmin):

    
    def sync_rss(modeladmin, request, queryset):
        for item in queryset:
            item.sync_rss()
    sync_rss.short_description = "Sync RSS"


    core_fields = (
        ('title','slug'),
        'url',
        'last_imported',
        'active',
        'logo_url',
        ('logo_width','logo_height'),
        'regex'
    )
    fieldsets = (
        ("Main Body", {
            'fields': core_fields,
            'classes': ( 'grp-collapse grp-open', )
        }),
        
        ("Meta", {
            'fields': BaseVersionableTitleAdmin.meta_fields,
            'classes': ( 'grp-collapse grp-closed', )
        })
    )

    list_display = ['title', 'url', 'last_imported', 'active']
    actions = [sync_rss]



import pytz

from django.db import models
from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from django.db.models.signals import pre_delete

from carbon.atoms.models.content import TagMolecule, CategoryMolecule
from carbon.atoms.models.media import *

from imagekit import ImageSpec
from imagekit.models import ImageSpecField
from imagekit.models import ProcessedImageField
from imagekit.admin import AdminThumbnail
from imagekit.processors import ResizeToFill, ResizeToFit

from imagekit_cropper.processors import PositionCrop, PositionAndFormatCrop, FormatProcessor




class MediaPositionCrop(PositionCrop):
    def get_hash(self):
        #Hash based on crop value
        crop_value = getattr(self.image_instance, self.crop_position_field)
        file_modified_date = getattr(self.image_instance, 'file_modified_date')
        file_modified_timezone = '' if not file_modified_date else file_modified_date
        hashed = u"%s-%s-%s-%s-%s"%(self.crop_position_field, crop_value, self.width, self.height, file_modified_timezone)
        # print hashed
        return hashed


class MediaPositionAndFormatCrop(PositionAndFormatCrop):
    
    def get_hash(self):
        #Hash based on crop value
        crop_value = getattr(self.image_instance, self.crop_position_field)
        file_modified_date = getattr(self.image_instance, 'file_modified_date')
        file_modified_timezone = '' if not file_modified_date else file_modified_date
        format = getattr(self.image_instance, self.format_field)
        hashed = u"%s-%s-%s-%s-%s-%s"%(self.crop_position_field, crop_value, format, self.width, self.height, file_modified_timezone)
        # print hashed
        return hashed

class MediaFormatProcessor(FormatProcessor):
    
    def get_hash(self):
        #Hash based on format value
        file_modified_date = getattr(self.image_instance, 'file_modified_date')
        file_modified_timezone = '' if not file_modified_date else file_modified_date
        format = getattr(self.image_instance, self.format_field)
        hashed = u"%s-%s-%s-%s"%(format, self.width, self.height, file_modified_timezone)
        # print hashed
        return hashed     





class MediaTag(TagMolecule):
    publish_by_default = True
    # default_template = 'media_tag'
    # item_classes = [Image, Media, SecureImage, SecureMedia]
    
    class Meta:
        abstract = True

class MediaFolder(CategoryMolecule):
    publish_by_default = True
    # item_class = None
    # item_classes = None
    # category_property_name = 'folder'

    
    class Meta:
        abstract = True


class Image(ImageMolecule):

    #EXAMPLE VARIANTS
    # variants = ('thumbnail','width_1200', 'width_1200_fill', 'square_600')

    # square_600 = ImageSpecField( source='image', format='PNG',
    #     processors=[ResizeToFill(600, 600)], options={'quality': 85})

    # width_1200 = ImageSpecField( source='image', format='PNG',
    #     processors=[ResizeToFit(1200, None, False)], options={'quality': 85})

    # width_1200_fill = ImageSpecField( source='image', format='PNG',
    #     processors=[ResizeToFit(1200, None)], options={'quality': 85})

    # tags = models.ManyToManyField('media.MediaTag', blank=True, related_name='%(app_label)s_%(class)s_tags')

    class Meta:
        abstract = True

class SecureImage(SecureImageMolecule):

    #EXAMPLE VARIANTS
    # variants = ('thumbnail','width_1200', 'width_1200_fill', 'square_600')

    # square_600 = ImageSpecField( source='image', format='PNG',
    #     processors=[ResizeToFill(600, 600)], options={'quality': 85})

    # width_1200 = ImageSpecField( source='image', format='PNG',
    #     processors=[ResizeToFit(1200, None, False)], options={'quality': 85})

    # width_1200_fill = ImageSpecField( source='image', format='PNG',
    #     processors=[ResizeToFit(1200, None)], options={'quality': 85})

    # tags = models.ManyToManyField('media.MediaTag', blank=True, related_name='%(app_label)s_%(class)s_tags')

    class Meta:
        abstract = True        




class Media(MediaMolecule):

    # tags = models.ManyToManyField('media.MediaTag', blank=True, related_name='%(app_label)s_%(class)s_tags')

    class Meta:
        abstract = True
        verbose_name_plural = 'media'



class SecureMedia(SecureMediaMolecule):

    # tags = models.ManyToManyField('media.MediaTag', blank=True, related_name='%(app_label)s_%(class)s_tags')

    class Meta:
        abstract = True
        verbose_name_plural = 'secure media'




@receiver(pre_delete, sender=Image, dispatch_uid='image_delete_signal')
def on_delete_image(sender, instance, using, **kwargs):
    print 'on delete image test'
    instance.image.delete(save=False)  

@receiver(pre_delete, sender=Media, dispatch_uid='media_delete_signal')
def on_delete_media(sender, instance, using, **kwargs):
    instance.file.delete(save=False)  

@receiver(pre_delete, sender=SecureMedia, dispatch_uid='secure_media_delete_signal')
def on_delete_secure_media(sender, instance, using, **kwargs):
    instance.file.delete(save=False) 

@receiver(pre_delete, sender=SecureImage, dispatch_uid='secure_image_delete_signal')
def on_delete_secure_media(sender, instance, using, **kwargs):
    instance.image.delete(save=False) 


# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import datetime

import django.http
import django.template
from django.conf import settings

from django.utils.cache import patch_vary_headers

# http://wtanaka.com/django/django403
class Django403Middleware(object):
  """Replaces vanilla django.http.HttpResponseForbidden() responses
  with a rendering of 403.html
  """
  def process_response(self, request, response):
    # If the response object is a vanilla 403 constructed with
    # django.http.HttpResponseForbidden() then call our custom 403 view
    # function
    if isinstance(response, django.http.HttpResponseForbidden) and \
        set(dir(response)) == set(dir(django.http.HttpResponseForbidden())):
      
      try:
        return access_denied(request)


      except django.template.TemplateDoesNotExist, e:
        return fallback_403(request)

    return response

def access_denied(request, template_name='403.html'):
  """
  Default 403 handler, which looks for the  which prints out a hard-coded string patterned
  after the Apache default 403 page.

  Templates: `403.html`
  Context:
      request
          The django request object
  """
  t = django.template.loader.get_template(template_name)
  template_values = {}
  template_values['request'] = request
  return django.http.HttpResponseForbidden(
      t.render(django.template.RequestContext(request, template_values)))


def fallback_403(request):
  """
  Fallback 403 handler which prints out a hard-coded string patterned
  after the Apache default 403 page.

  Templates: None
  Context: None
  """
  return django.http.HttpResponseForbidden(
      django.utils.translation.gettext(
          """<!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML 2.0//EN">
<html><head>
<title>403 Forbidden</title>
</head><body>
<h1>Forbidden</h1>
<p>You don't have permission to access %(path)s on this server.</p>
<hr>
</body></html>""") % {'path': request.path})

def access_denied(request, template_name='403.html'):
  """
  Default 403 handler, which looks for the  which prints out a hard-coded string patterned
  after the Apache default 403 page.

  Templates: `403.html`
  Context:
      request
          The django request object
  """
  t = django.template.loader.get_template(template_name)
  template_values = {}
  template_values['request'] = request
  return django.http.HttpResponseForbidden(
      t.render(django.template.RequestContext(request, template_values)))



#https://pypi.python.org/pypi/django-donottrack/0.1
def set_cookie(response, key, value, days_expire = 7):
  if days_expire is None:
    max_age = 365 * 24 * 60 * 60  #one year
  else:
    max_age = days_expire * 24 * 60 * 60 
  expires = datetime.datetime.strftime(datetime.datetime.utcnow() + datetime.timedelta(seconds=max_age), "%a, %d-%b-%Y %H:%M:%S GMT")
  response.set_cookie(key, value, max_age=max_age, expires=expires, domain=settings.SESSION_COOKIE_DOMAIN, secure=settings.SESSION_COOKIE_SECURE or None)


def get_donottrack(request):
    """
    Returns ``True`` if ``HTTP_DNT`` header is ``'1'``, ``False`` otherwise.

    """
    return request.META.get('HTTP_DNT') == '1'

class DoNotTrackMiddleware(object):
    """
    Middleware that inspects the ``HTTP_DNT`` header and provides information
    for your app to act appropriately.

    """

    def process_request(self, request):
        """
        Inspects the request for the ``HTTP_DNT`` header and sets a
        ``donottrack`` attribute on the request object appropriately.

        Doing this here, rather than in a context processor, allows for your
        views to also take advantage of this logic.

        """
        request.donottrack = get_donottrack(request)



        return None

    def process_response(self, request, response):
        """
        Adds a vary header for ``DNT``, allowing for cache control based on the
        ``HTTP_DNT`` header.

        """
        patch_vary_headers(response, ('DNT',))
        if hasattr(request, 'donottrack') and request.donottrack:
          response.set_cookie("donottrack", '1')
        else:
          response.delete_cookie("donottrack")


        return response

class SiteMiddlewear(object):
    def process_request(self, request):
        try:

            request.site = Site.objects.get_current()
        except:
            request.site = {
                'domain': '',
                'name': settings.GRAPPELLI_ADMIN_TITLE,
            }



class CustomSettingsMiddlewear(object):
    def process_request(self, request):

        try:

            all_settings = settings.CUSTOM_SETTINGS
            for setting in all_settings:
                if hasattr(settings, setting):
                    value = getattr(settings, setting)
                    setattr(request, setting, value)
                    #print "Added %s to request %s"%(setting, getattr(request, setting))
        except:
            pass

class ImpersonateMiddleware(object):
    def process_request(self, request):
        request.impersonating = None
        request.original_user = None
        
        if hasattr(request, 'user') and request.user and request.user.is_authenticated() and  request.user.is_superuser and "__impersonate" in request.GET:
            request.session['impersonate_id'] = int(request.GET["__impersonate"])
        elif "__unimpersonate" in request.GET:
            if "impersonate_id" in request.session:
                del request.session['impersonate_id']

            if '__unimpersonate' in request.GET:
                new_query = request.GET.copy()
                del new_query['__unimpersonate']
                new_path = "%s?%s"%(request.path, new_query.urlencode())
                return HttpResponseRedirect(new_path)
        
        if hasattr(request, 'user') and request.user and request.user.is_authenticated() and request.user.is_superuser and 'impersonate_id' in request.session:
            try:
                UserModel = get_user_model()
                user = UserModel._default_manager.get(pk=request.session['impersonate_id'])
                request.original_user = request.user
                request.impersonating = request.user = user   
                

                if '__impersonate' in request.GET:
                    new_query = request.GET.copy()
                    del new_query['__impersonate']
                    new_path = "%s?%s"%(request.path, new_query.urlencode())
                    return HttpResponseRedirect(new_path)


            except:
                pass


class AdminLoggedInCookieMiddlewear(object):
    def process_response(self, request, response):

        is_staff = hasattr(request, 'user') and request.user and request.user.is_authenticated() and request.user.is_staff
        if is_staff and not request.COOKIES.get('admin_logged_in'):
            response.set_cookie("admin_logged_in", '1')
        elif not is_staff and request.COOKIES.get('admin_logged_in'):
            response.delete_cookie("admin_logged_in")
        return response


class SiteVersionMiddleware(object):

    def get_heroku_release_version(self):
        try:
            import heroku
            key = settings.HEROKU_API_KEY
            cloud = heroku.from_key(key)
            app = cloud.apps[settings.HEROKU_APP_LABEL]            
            latest_release = app.releases[-1]
            return latest_release.name
        except:
            return None

    def get_git_release_version(self):
        try:
            import subprocess
            short_revision = subprocess.check_output(['git', 'rev-parse', '--short', 'HEAD']).replace('\n','')
            return '@%s'%(short_revision)
        except:
            return None

    def process_request(self, request):
        if hasattr(request, 'user') and request.user and request.user.is_authenticated() and request.user.is_staff: 
            if settings.IS_ON_SERVER:
                if settings.IS_ON_HEROKU:
                    setattr(request, "RELEASE_VERSION", self.get_heroku_release_version())
                else:
                    setattr(request, "RELEASE_VERSION", self.get_git_release_version())
            else:
                git_version = self.get_git_release_version()
                heroku_version = self.get_heroku_release_version()

                if git_version and heroku_version:
                    version = '%s %s'%(git_version, heroku_version)
                elif git_version:
                    version = git_version
                elif heroku_version:
                    version = heroku_version
                else:
                    version = "UNKNOWN"

                setattr(request, "RELEASE_VERSION", version)


from django.conf import settings
from django.contrib import messages
from django.core.cache import cache
from django.core.exceptions import ValidationError
from django.core.urlresolvers import reverse
from django.core.validators import URLValidator
from django.http import HttpResponse, Http404, HttpResponseServerError, HttpResponsePermanentRedirect
from django.shortcuts import redirect, render_to_response
from django.template import RequestContext
from django.utils import timezone
from django.utils.translation import ugettext as _

from grappelli.views.related import AutocompleteLookup

#ADMIN VIEWS 








def clear_cache(request):
    
    if not request.user or not request.user.is_staff:
        raise Http404()

    try:
        cache.clear()
        messages.success(request, 'The cache has been cleared.')
    except:
        messages.warning(request, 'There was an error while attempting to clear the cache.')

    return redirect('admin:index')


class FilteredAutocomplete(AutocompleteLookup):
    """ patch grappelli's autocomplete to let us control the queryset 
    by creating a autocomplete_queryset function on the model """
    def get_queryset(self):
        if hasattr(self.model, "autocomplete_queryset"):
            qs = self.model.autocomplete_queryset()
        else:
            qs = self.model._default_manager.all()
        qs = self.get_filtered_queryset(qs)
        qs = self.get_searched_queryset(qs)
        return qs.distinct()    


from django.db import models
from django.conf import settings
from django.utils.translation import ugettext_lazy as _

from carbon.atoms.models.abstract import VersionableAtom, TitleAtom, OrderedItemAtom
from carbon.atoms.models.content import ContentMolecule
from carbon.atoms.models.access import AccessAtom
from carbon.atoms.models.user import UserMolecule, StreetAddressMolecule, \
    UserProfileMolecule, SocialContactLinkMolecule

from .manager import UserManager

class User(UserProfileMolecule):

    objects = UserManager()

    class Meta:
        abstract = True


class Address(StreetAddressMolecule):

    class Meta:
        abstract = True

class UserGroup(ContentMolecule):

    def get_members(self):
        if not self.member_class:
            raise NotImplementedError('Class should specify an member_class')
        
        return self.member_class.objects.filter(group=self).order_by('order')

    class Meta:
        abstract = True
        ordering = ['order']

class UserGroupMember(VersionableAtom, OrderedItemAtom):

    class Meta:
        abstract = True
        ordering = ['order']

    # YOU MUST IMPLEMENT THIS:
    # group = models.ForeignKey('account.UserGroup', 
    #     blank=True, null=True)

    user = models.ForeignKey(settings.AUTH_USER_MODEL, 
        blank=True, null=True)


class Organization(ContentMolecule):

    class Meta:
        abstract = True

    creator = models.ForeignKey(settings.AUTH_USER_MODEL, 
        blank=True, null=True)

    def get_admins(self):
        return OrganizationMember.objects.filter(organization=self,access=AccessAtom.OWNER)

    def get_editors(self):
        return OrganizationMember.objects.filter(organization=self,access=AccessAtom.WRITE)

    def get_viewers(self):
        return OrganizationMember.objects.filter(organization=self,access=AccessAtom.READ)

class OrganizationMember(VersionableAtom, AccessAtom):

    class Meta:
        abstract = True

    # YOU MUST IMPLEMENT THIS:
    # organization = models.ForeignKey('Organization', 
    #     blank=True, null=True)

    user = models.ForeignKey(settings.AUTH_USER_MODEL, 
        blank=True, null=True)

class SocialContactLink(SocialContactLinkMolecule):
    class Meta:
        abstract = True


from django.db import models
from django.conf import settings
from django.core.urlresolvers import reverse_lazy
from django.utils.translation import ugettext_lazy as _

from bs4 import BeautifulSoup

from carbon.utils.slugify import unique_slugify
from carbon.atoms.models.abstract import *
from carbon.atoms.models.content import *

#TODO
# class ContentBlock():
#     pass

class Page(HierarchicalAtom, ContentMolecule):

    

    def get_url_path(self):
        path = self.path
        if path.startswith('/'):
            path = path[1:]

        # if path.endswith('/'):
        #     path = path[:1]
            
        return path

    def get_absolute_url(self):
        if self.is_external:
            return self.path_override

        if hasattr(self.__class__, 'detail_view') and self.__class__.detail_view:
            return reverse_lazy(self.__class__.detail_view, kwargs = {'path': self.get_url_path() })   

        return reverse_lazy('page_page', kwargs = {'path': self.get_url_path() })   

    class Meta:
        abstract = True



class PageTag(TagMolecule):  
    publish_by_default = True

    class Meta:
        verbose_name_plural = 'Page Tags'

    def get_absolute_url(self):
        if self.is_external:
            return self.path_override
        return reverse_lazy('page_tag', kwargs = {'path': self.get_url_path() })   

    def get_children(self):
        all_children = self.__class__.objects.filter(parent=self)
        return [child for child in all_children if child.is_published()]

    class Meta:
        abstract = True
       
class PageContentBlock(VersionableAtom, OrderedItemAtom, TitleAtom, ContentAtom, PublishableAtom):
    
    publish_by_default = True

    parent = models.ForeignKey('page.Page', related_name="page_blocks")

    def get_siblings(self, published=True):
        siblings = self.__class__.objects.filter(parent=self.parent)

        if published:
            return [child for child in siblings if child.is_published()]
        else:
            return siblings

    def verify_title_and_slug(self):
        siblings = self.get_siblings(False)

        if not self.title:
            self.title = '%s %s'%(self._meta.verbose_name.title(), len(siblings)+1)

        if not self.slug or not self.pk:
            self.slug = self.generate_slug()

    def generate_slug(self):
        
        siblings = self.get_siblings(False)
        # -- Make sure slug is unique
        if self.slug:
            raw_slug = self.slug
        else:
            raw_slug = self.title

        raw_slug = raw_slug.lower()
        
        unique_slugify(self, raw_slug, 'slug', siblings, "_")

        return self.slug

    class Meta:
        verbose_name = 'Page Block'
        verbose_name_plural = 'Page Blocks'
        ordering = ['order']
        abstract = True


class GlobalContentBlock(VersionableAtom, TitleAtom, ContentAtom, PublishableAtom):
    
    publish_by_default = True


    class Meta:
        verbose_name = 'Global Content Block'
        verbose_name_plural = 'Global Content Blocks'
        abstract = True        
    
    


"""
This file was generated with the customdashboard management command and
contains the class for the main dashboard.

To activate your index dashboard add the following to your settings.py::
    GRAPPELLI_INDEX_DASHBOARD = 'project.dashboard.CustomIndexDashboard'
"""

from django.utils.translation import ugettext_lazy as _
from django.core.urlresolvers import reverse

from grappelli.dashboard import modules, Dashboard
from grappelli.dashboard.modules import DashboardModule
from grappelli.dashboard.utils import get_admin_site_name




# class AdminTasksDashboardModule(DashboardModule):

#     title = 'Admin Tasks'
#     template = 'admin/admin_tasks.html'
    
#     def getActionMessage(self, item_count):
#         if item_count == 0:
#             return ""
#         else:
#             return "There are new admin tasks."

    
#     def getVerbPhrase(self, count):
#         if count >= 1:
#             return str(count)
#         else:
#             return "No"

#     def init_with_context(self, context):
#         if self._initialized:
#             return
#         new_children = []
        
#         unconnected_urls = LegacyURL.objects.needs_redirect()
#         if len(unconnected_urls) > 0:
#             new_children.append({
#                 'title': "There are %s urls to be redirected"%(self.getVerbPhrase(len(unconnected_urls))), 
#                 'url': "/admin/site_admin/legacyurl/?_redirect_path=0",
#                 "external":False,
#             })

#         total_count = len(unconnected_urls)

#         self.admin_message = self.getActionMessage(total_count)
#         self.children = new_children
#         self._initialized = True


class BaseAdminDashboard(Dashboard):
    
    #sidebar_model
    #main_model
    

    def init_with_context(self, context):
        site_name = get_admin_site_name(context)

        main_link_sets = self.main_model.objects.all()
        for link_set in main_link_sets:

            children = link_set.get_children()
            child_links = [link.model_path for link in children]
            # -- Django Admin

            classes = ['grp-closed'] if not link_set.open_by_default else []
            self.children.append(modules.AppList(
                title=_(link_set.title),
                column=1,
                collapsible=True,
                models=child_links,
                css_classes=classes
            ))
        
        
        classes = ['grp-closed'] if main_link_sets.count()>0 else []

        # -- Django Admin
        self.children.append(modules.AppList(
            title=_('All Models'),
            column=1,
            collapsible=True,
            css_classes=classes
        ))

         

        sidebar_link_sets = self.sidebar_model.objects.all()
        for link_set in sidebar_link_sets:

            children = []
            child_links = link_set.get_children()
            for child_link in child_links:
                children.append({
                    'title':child_link.title,
                    'url':child_link.url,
                    'external':False
                })

            # append another link list module for "support".
            self.children.append(modules.LinkList(
                link_set.title,
                column=2,
                children=children
            ))
        

        
        # self.children.append(AdminTasksDashboardModule(

        #     _('Admin Tasks'),
        #     column=2,

        # ))

        # append a recent actions module
        self.children.append(modules.RecentActions(
            _('Recent Actions'),
            limit=5,
            collapsible=False,
            column=2,
        ))





from django.contrib import admin

from carbon.atoms.admin.content import *
from carbon.atoms.admin.taxonomy import *

from reversion.admin import VersionAdmin

from .models import *
from .forms import *

from django_inline_wrestler.admin import TabularInlineOrderable


class BlogArticleRoleInline(TabularInlineOrderable):
    #model = BlogArticleRole
    autocomplete_lookup_fields = {
        'fk': ['user',],
    }
    raw_id_fields = ('user',)

    fields = ('order', 'user', 'role')
    

    sortable_field_name = 'order'
    extra = 0    


class BlogArticleAdmin(VersionAdmin, BaseContentAdmin):
    form = BlogArticleAdminForm

    autocomplete_lookup_fields = {
        'fk': ('image', 'published_by', 'template', 'category'),
        'm2m': ('tags','related')
    }
    raw_id_fields = ( 'image', 'published_by', 'template','category', 'tags', 
        'related')
    

    core_fields = BaseContentAdmin.core_fields
    core_fields_list = list(core_fields)
    core_fields_list.insert(5, 'category')
    core_fields_list.insert(5, 'tags')    
    core_fields_list.insert(5, 'related')    
    core_fields = tuple(core_fields_list)

    path_fields = BaseContentAdmin.path_fields
    publication_fields = BaseContentAdmin.publication_fields
    seo_fields = BaseContentAdmin.seo_fields
    
    social_fields = BaseContentAdmin.social_fields
    social_fields_list = list(social_fields)
    social_fields_list.insert(1, 'allow_comments')
    social_fields = tuple(social_fields_list)
    

    meta_fields = BaseVersionableAdmin.meta_fields

    fieldsets = (
        ("Main Body", {
            'fields': core_fields,
            'classes': ( 'grp-collapse grp-open', )
        }),
        ("Roles", {
            'fields': (),
            'classes': ( 'placeholder blogarticlerole_set-group', )
        }),
        ("Related", {
            'fields': (),
            'classes': ( 'placeholder blogrelatedobject_set-group', )
        }),
        ("Path", {
            'fields': path_fields,
            'classes': ( 'grp-collapse grp-closed', )
        }),
        ("Publication", {
            'fields': publication_fields,
            'classes': ( 'grp-collapse grp-closed', )
        }),
        ("Search Engine Optimization", {
            'fields': seo_fields,
            'classes': ( 'grp-collapse grp-closed', )
        }),
        ("Social Integration", {
            'fields': social_fields,
            'classes': ( 'grp-collapse grp-closed', )
        }),
        ("Meta", {
            'fields': meta_fields,
            'classes': ( 'grp-collapse grp-closed', )
        })
    )

class BlogTagAdmin(BaseTagAdmin):
    form = BlogTagAdminForm

class BlogCategoryAdmin(BaseCategoryAdmin):
    form = BlogCategoryAdminForm    





class BlogCommentVoteFlagInline(admin.TabularInline):
    #model = BlogCommentVote or BlogCommentFlag
    autocomplete_lookup_fields = {
        'fk': ['voter'],
    }
    raw_id_fields = ('comment','voter',)
    fields = ('voter', 'type')
    extra = 0

class BlogCommentAdmin(VersionAdmin, BaseContentAdmin):
    

    list_display = ( "pk", "title", "user", "article", "in_response_to", "moderation_status", "is_deleted", "created_date")
    list_display_links = ( "pk", "title", )
    list_filter = ("moderation_status", "user",)

    readonly_fields = (
        "version", "created_date", "created_by", "modified_date", "modified_by",
    )
    
    autocomplete_lookup_fields = {
        'fk': ('article','in_response_to', 'user'),
    }
    raw_id_fields = ( 'article','in_response_to', 'user')
    

    core_fields =  (
        ('moderation_status', ),
        ('article', 'in_response_to',),
        ('user', 'is_deleted',),
        ('title', 'slug'),
        'content',
        'moderation_comment',
        'cleaned_content'
    )

    meta_fields = BaseVersionableAdmin.meta_fields

    fieldsets = (
        ("Main Body", {
            'fields': core_fields,
            'classes': ( 'grp-collapse grp-open', )
        }),
        ("Meta", {
            'fields': meta_fields,
            'classes': ( 'grp-collapse grp-closed', )
        })
    )



# admin.site.register(BlogArticle, BlogArticleAdmin)
# admin.site.register(BlogTag, BlogTagAdmin)
# admin.site.register(BlogCategory, BlogCategoryAdmin)
# admin.site.register(BlogComment, BlogCommentAdmin)


from django.contrib.auth.models import BaseUserManager
from django.db import models
from django.utils import timezone


class UserQueryset(models.query.QuerySet):
    

    def active(self):
        return self.filter(is_active=True)


class UserManager(BaseUserManager):
    """
    ==================
    User Model Manager
    ==================

    Model manager for extending User Model methods.
    """

    def create_user(self, username=None, email=None, password=None, **extra_fields):
        """
        Creates and saves a User with the given username, email and password.
        """
        now = timezone.now()
        email = self.normalize_email(email)
        user = self.model(
            email=email,
            is_staff=False,
            is_active=True,
            is_superuser=False,
            **extra_fields
        )
        user.set_password(password)
        user.save(using=self._db)
        return user

    def create_superuser(self, email, password, **extra_fields):
        """
        Create a Super User using `create_user` and overriding permissions.
        """
        u = self.create_user(
            email,
            email,
            password,
            **extra_fields
        )
        u.is_staff = True
        u.is_active = True
        u.is_superuser = True
        u.save(using=self._db)
        return u

    def get_queryset(self):
        return UserQueryset(self.model, using=self._db)

    def active(self):
        return self.get_queryset().active()



from django.apps import AppConfig
from django.conf import settings

class CoreConfig(AppConfig):
    name = settings.CORE_APP

    def ready(self):
        import carbon.compounds.core.signals.handlers


import json
from mimetypes import MimeTypes
import urllib

from django.contrib import admin
from django.contrib.admin import helpers
from django.contrib.admin.filters import SimpleListFilter
from django.contrib.admin.models import LogEntry, ADDITION
from django.contrib.contenttypes.models import ContentType
from django.contrib import messages
from django.http import HttpResponseRedirect, HttpResponse
from django.core.urlresolvers import reverse_lazy, resolve
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.utils.decorators import method_decorator
from django.utils.http import urlencode
from django.views.decorators.csrf import csrf_protect
from django.utils.safestring import mark_safe


from .content import *

csrf_protect_m = method_decorator(csrf_protect)



class TagListFilter(SimpleListFilter):
    title = 'tag'
    parameter_name = 'tag'
    def lookups(self, request, model_admin):

      tags = model_admin.tag_model.objects.all()
      items = ()
      for tag in tags:
        items += ((str(tag.id), str(tag.title),),)
      return items

    def queryset(self, request, queryset):
      tag_id = request.GET.get(self.parameter_name, None)
      if tag_id:
        return queryset.filter(tags=tag_id)
      return queryset

class FolderListFilter(SimpleListFilter):
    title = 'folder'
    parameter_name = 'folder'
    def lookups(self, request, model_admin):

        folders = model_admin.folder_model.objects.all()
        items = ()
        for folder in folders:
            items += ((str(folder.id), str(folder.title_path),),)

        items += ((str(-1), "Items in any or all folders",),)
        return items

    def queryset(self, request, queryset):
      folder_id = request.GET.get(self.parameter_name, None)
      if folder_id:
        return queryset.filter(folder=folder_id)
      return queryset


class BaseMedia(object):
    # autocomplete_lookup_fields = {
    #     'fk': ('folder'),
    #     'm2m': ('tags',)
    # }
    # raw_id_fields = ( 'tags','folder')

    
    search_fields = ('title', 'alt', 'caption', 'credit')
    change_list_template = "admin/media/change_list.html"

    @csrf_protect_m
    def changelist_view(self, request, extra_context=None):
        response = super(BaseMedia, self).changelist_view(request, extra_context)
    
        try:
            context_data = response.context_data
            context_data['extra_urls'] = self.extra_urls
        except:
            pass 
        
        return response



class FolderTagAdmin(admin.ModelAdmin):

    change_list_template = "admin/media/change_list_image_folders.html"
    
    def get_folder_qs(self, request, folder_id=None):
        if folder_id is None:
          folder_id = request.POST.get('folder__id__exact', None) or request.POST.get('folder_id', None) or request.POST.get('folder', None)

        if folder_id is None:
            folder_id = request.GET.get('folder__id__exact', None) or request.GET.get('folder_id', None) or request.GET.get('folder', None)

        if folder_id=="None":
            folder_id=None

        return folder_id

    def get_tag_qs(self, request, tag_id=None):
        if tag_id is None:
          tag_id = request.POST.get('tag__id__exact', None) or request.POST.get('tag_id', None) or request.POST.get('tag', None)

        if tag_id is None:
            tag_id = request.GET.get('tag__id__exact', None) or request.GET.get('tag_id', None) or request.GET.get('tag', None)

        return tag_id

    def get_folder(self, request, folder_id=None):
        
        folder_id = self.get_folder_qs(request, folder_id)

        if folder_id and int(folder_id) >= 0:

            return self.folder_model.objects.get(pk=folder_id)
        return None

    def get_tag(self, request, tag_id=None):
        
        tag_id = self.get_tag_qs(request, tag_id)

        if tag_id and int(tag_id) >= 0:

            return self.tag_model.objects.get(pk=tag_id)
        return None
    
    def get_add_folder_url(self, request):

        url = reverse('admin:%s_%s_add'%(self.folder_model._meta.app_label.lower(), self.folder_model._meta.object_name.lower()))  

        current_folder = self.get_folder(request)
        if current_folder:
            current_url = request.get_full_path()
            url = '%s?%s'%(url, urlencode({'parent': current_folder.pk, '_redirect_to':current_url}))

        return url

    def ignore_folders(self, request):
        has_query = request.GET.get('q', None) != None
        folder_id = self.get_folder_qs(request)
        if has_query and not folder_id:
            return True

        
        if folder_id and int(folder_id) < 0:
            return True

        return False

    def get_folders(self, current_folder=None):
        if current_folder:
            return self.folder_model.objects.filter(parent=current_folder)
        else:
            return self.folder_model.objects.filter(parent=None)

    def get_all_folders(self):
        return self.folder_model.objects.all()



    def get_all_tags(self):
        return self.tag_model.objects.all()

    def tag_list(self, obj):
        
        output = ''
        all_tags = obj.tags.all()
        if len(all_tags) > 1:
            output += '<strong>Tags: </strong>'

        elif len(all_tags) > 0:
            output += '<strong>Tag: </strong>'

        for tag in all_tags:
            output += ('<a href="?tags__id__exact=%s">%s</a> '%(tag.pk, tag.title))
        return mark_safe(output)

        

    def get_queryset(self, request):
        qs = super(FolderTagAdmin, self).get_queryset(request)
        
        if 'changelist' in resolve(request.path).url_name:
            ignore_folders = self.ignore_folders(request)
            folder = self.get_folder(request)

            if folder is None and ignore_folders is False:
                qs = qs.filter(folder=None)


        return qs

    def move_to_folder(self, request, queryset):        

        if request.POST.get('post', None):

            folder = self.get_folder(request, request.POST.get('folder', None))
            
            count = len(queryset)
            for item in queryset:
                item.folder = folder
                print 'set folder to %s on %s'%(item.folder, item)
                item.save()

            if folder:
                #Redirect to folder that items were moved to...
                get = request.GET.copy()
                get['folder__id__exact'] = folder.pk
                path = '%s?' % request.path
                for query, val in get.items():
                    path += '%s=%s&' % (query, val)
                redirect_url = path[:-1]  
                messages.success(request, u'%s items moved to folder %s'%(count, folder))

            else:
                redirect_url = request.get_full_path()
                messages.success(request, u'%s items moved to the top-level, out of all folders'%(count))

          
            return HttpResponseRedirect(redirect_url)

        else:

            return render_to_response('admin/media/move_to_folder.html', 
            {
                'queryset': queryset, 
                'action_checkbox_name': helpers.ACTION_CHECKBOX_NAME,
                'folders':self.get_all_folders()
            }, 
            context_instance=RequestContext(request))

        move_to_folder.short_description = "Move to folder..."

    def tag_items(self, request, queryset):        

        if request.POST.get('post', None):

            tag = self.get_tag(request, request.POST.get('tag', None))
            
            

            if tag:

                count = len(queryset)
                for item in queryset:
                    item.tags.add(tag)
                    item.save()

                messages.success(request, u'%s items tagged with %s'%(count, tag))
          
            redirect_url = request.get_full_path()
            return HttpResponseRedirect(redirect_url)

        else:

            return render_to_response('admin/media/tag_items.html', 
            {
                'queryset': queryset, 
                'action_checkbox_name': helpers.ACTION_CHECKBOX_NAME,
                'tags':self.get_all_tags()
            }, 
            context_instance=RequestContext(request))

        tag_items.short_description = "Tag items..."

    def changelist_view(self, request, extra_context=None):
        extra_context = extra_context or {}
        current_folder = self.get_folder(request)
        extra_context['folders'] = self.get_folders(current_folder)
        extra_context['current_folder'] = current_folder
        extra_context['add_folder_url'] = self.get_add_folder_url(request)
        extra_context['ignore_folders'] = self.ignore_folders(request)
        return super(FolderTagAdmin, self).changelist_view(request, extra_context=extra_context)


    autocomplete_lookup_fields = {
        'fk': ('folder',),
        'm2m': ('tags',)
    }
    raw_id_fields = ( 'tags','folder')

    list_filter = BaseVersionableAdmin.list_filter + (TagListFilter, FolderListFilter,)
    actions = [move_to_folder, tag_items]

    readonly_fields = (
        "version", "created_date", "created_by", "modified_date", "modified_by",
        "display_size", "size", "dimensions", "tag_list"
    )

class BaseImageAdmin(BaseMedia, BaseVersionableAdmin):

    change_list_template = "admin/media/change_list_image.html"

    def preview(self, obj):
        if obj.image:
            try:
                return mark_safe("<img src='%s' alt='%s preview'/>"%(obj.thumbnail.url, obj.title))
            except:
                return ""
        return ''

    

    def image_variants(self, obj):
        if obj.image:
            base_image =  '<a href="%s" target="_blank">Original Size (%spx x %spx)</a><br />'%(obj.image_url, obj.image_width, obj.image_height)
            for variant in obj.__class__.variants:
                image_variant_name = variant.replace("_", " ").title()
                base_image +=  '<a href="%s" target="_blank">%s (%spx x %spx)</a><br />'%(obj.get_variant_url(variant), image_variant_name, obj.get_variant_width(variant), obj.get_variant_height(variant))

            return mark_safe(base_image)
    

    def image_variants_links(self, obj):

        if obj.image:
            base_image =  '<a href="%s" target="_blank">Original Size (%spx x %spx)</a><br />'%(obj.image_url, obj.image_width, obj.image_height)
            for variant in obj.__class__.variants:
                image_variant_name = variant.replace("_", " ").title()
                base_image +=  '<a href="%s" target="_blank">%s</a><br />'%(obj.get_variant_url(variant), image_variant_name)

            return mark_safe(base_image)
    

    def dimensions(self, obj):
        if obj.image:
            return mark_safe('Original Dimensions: %sx%s Size: %s'%(obj.image_width, obj.image_height, obj.display_size))
            return mark_safe(base_image)


    readonly_fields = (
        "version", "created_date", "created_by", "modified_date", "modified_by",
        "preview", "image_variants", "image_width", "image_height",
        "display_size", "size", "dimensions", "file_modified_date"
    )
    
    core_fields = (
        'title',
        ('image','preview'),
        ('dimensions'),
        ('image_variants'),
        ('clean_filename_on_upload','allow_overwrite'),
        ('alt','use_png'),
        'credit',
        'caption'
    )
    
    meta_fields = (
        ('version',),
        ('created_date', 'created_by'),
        ('modified_date', 'modified_by'),
        'file_modified_date',
        'admin_note'
    )
    

    fieldsets = (
        ("Image", {
            'fields': core_fields,
        }),
        ("Meta", {
            'fields': meta_fields,
            'classes': ( 'grp-collapse grp-closed', )
        })
    )

    list_display = ('title','preview','image_width', 'image_height', 'display_size',)
    list_display_links = ('title', 'preview')



class BaseMediaAdmin(BaseMedia, BaseVersionableAdmin):


    readonly_fields = (
        "version", "created_date", "created_by", "modified_date", "modified_by",
        "file_modified_date"
    )   

    
    core_fields = (
        'title',
        ('file',),
        ('clean_filename_on_upload','allow_overwrite'),
        'credit',
        'caption'
    )

    meta_fields = (
        ('version',),
        ('created_date', 'created_by'),
        ('modified_date', 'modified_by'),
        'file_modified_date',
        'admin_note'
    )

    fieldsets = (
        ("Image", {
            'fields': core_fields,
        }),
        ("Meta", {
            'fields': meta_fields,
            'classes': ( 'grp-collapse grp-closed', )
        })
    )
    list_display = ('title','file','display_size', 'size')
    

class BaseSecureMediaAdmin(BaseMediaAdmin):
    def secure_url(self, obj):
        duration = 120
        url = obj.get_secure_url(duration)
        if url:
            return mark_safe("<a href='%s'>Download</a><br /><br />This link will expire after %s seconds.</a>"%(url, duration))
        return ''

    readonly_fields = (
        "version", "created_date", "created_by", "modified_date", "modified_by",
        "secure_url"
    )  
    core_fields = (
        'title',
        ('file','secure_url'),
        ('clean_filename_on_upload','allow_overwrite'),
        'credit',
        'caption',
    )
    meta_fields = (
        ('version',),
        ('created_date', 'created_by'),
        ('modified_date', 'modified_by'),
        'file_modified_date',
        'admin_note'
    )

    fieldsets = (
        ("Image", {
            'fields': core_fields,
        }),
        ("Meta", {
            'fields': meta_fields,
            'classes': ( 'grp-collapse grp-closed', )
        })
    )





from .widgets import *

class FieldData(object):

	def __init__(self, is_required, secondary_label='', placeholder='', help_text=''):
		self.is_required = is_required
		self.secondary_label = secondary_label
		self.placeholder = placeholder
		self.help_text = help_text

		self.icon = None
		self.extra_css_classes = None
		self.max_length = None
		self.pattern = None
		self.inset_text_right = None
		self.inset_text_left = None
		self.icon_right = None
		self.icon_left = None

def get_form_field_input_type_by_type(type):
	from .models import FormField
	if type == FormField.TEXT_FIELD:
		return 'text'

	if type == FormField.EMAIL_FIELD:
		return 'email'

	if type == FormField.URL_FIELD:
		return 'url'

	if type == FormField.INTEGER_FIELD:
		return 'number'

	if type == FormField.NUMBER_FIELD:
		return 'number'

	if type == FormField.HIDDEN_FIELD:
		return 'hidden'

	if type == FormField.HONEYPOT_FIELD:
		return 'text'

	elif type == FormField.TEXT_AREA:
		return 'text' #TODO

	elif type == FormField.BOOLEAN_CHECKBOXES:
		return 'checkbox'

	elif type == FormField.BOOLEAN_TOGGLE:				
		return 'checkbox'

	elif type == FormField.SELECT_DROPDOWN:
		return 'select'

	elif type == FormField.SELECT_RADIO_BUTTONS:
		return 'radio'

	elif type == FormField.SELECT_BUTTONS:
		return 'radio'

	elif type == FormField.SELECT_IMAGE:
		return 'radio'

	elif type == FormField.SELECT_MULTIPLE_CHECKBOXES:
		return 'checkbox'

	elif type == FormField.SELECT_MULTIPLE_AUTOSUGGEST:
		return 'select' #TODO

	elif type == FormField.SELECT_MULTIPLE_HORIZONTAL:
		return 'select' #TODO

	elif type == FormField.SELECT_MULTIPLE_IMAGES:
		return 'checkbox'

	elif type == FormField.COMMA_SEPARATED_LIST:
		return 'text'
	
	elif type == FormField.FILE:
		return 'file'

	elif type == FormField.SECURE_FILE:
		return 'file'

	elif type == FormField.IMAGE:
		return 'file'

	elif type == FormField.DATE:
		return 'text'

	elif type == FormField.TIME:
		return 'text'

	elif type == FormField.DATE_TIME:
		return 'text'

	elif type == FormField.SCORE:
		return 'number'

	elif type == FormField.RANGE:
		return 'range'

	elif type == FormField.NUMBER_SLIDER:
		return 'number'

	elif type == FormField.PASSWORD:
		return 'password'

	return 'text'


def get_form_field_by_type(type, data, label='', choices=None):
	from .models import FormField
	"""
	data should provide:
	- is_required:Boolean
	- secondary_label
	- placeholder
	- help_text

	Optional:
	
	- icon:font awesome class
	- extra_css_classes: char
	- max_length
	- pattern
	- inset_text_right
	- inset_text_left
	- icon_right: font awesome
	- icon_left: font awesome
	"""

	field = None

	if type == FormField.TEXT_FIELD:
		field = forms.CharField(widget=SingleLineText(model_field=data), label=label)

	elif type == FormField.EMAIL_FIELD:
		field = forms.EmailField(widget=SingleLineText(model_field=data), label=label)

	elif type == FormField.URL_FIELD:
		field = forms.CharField(widget=SingleLineText(model_field=data), label=label)

	elif type == FormField.INTEGER_FIELD:
		field = forms.IntegerField(widget=SingleLineText(model_field=data), label=label)

	elif type == FormField.NUMBER_FIELD:
		field = forms.DecimalField(widget=SingleLineText(model_field=data), label=label)

	elif type == FormField.TEXT_AREA:
		field = forms.CharField(widget=MultiLineText(model_field=data), label=label)

	elif type == FormField.BOOLEAN_CHECKBOXES:
		field = forms.BooleanField(widget=Checkbox(model_field=data), label=label)

	elif type == FormField.BOOLEAN_TOGGLE:				
		field = forms.BooleanField(widget=Checkbox(model_field=data), label=label)

	elif type == FormField.SELECT_DROPDOWN:
		field = forms.ChoiceField(widget=Select(model_field=data,choices=choices), label=label, choices=choices)

	elif type == FormField.SELECT_RADIO_BUTTONS:
		field = forms.ChoiceField(widget=Select(model_field=data,choices=choices), label=label, choices=choices)

	elif type == FormField.SELECT_BUTTONS:
		field = forms.ChoiceField(widget=Select(model_field=data,choices=choices), label=label, choices=choices)

	elif type == FormField.SELECT_IMAGE:
		field = forms.ChoiceField(widget=Select(model_field=data,choices=choices), label=label, choices=choices)

	elif type == FormField.SELECT_MULTIPLE_CHECKBOXES:
		field = MultipleChoiceField(widget=SelectMultiple(model_field=data), label=label, choices=choices)

	elif type == FormField.SELECT_MULTIPLE_AUTOSUGGEST:
		field = MultipleChoiceField(widget=SelectMultiple(model_field=data), label=label, choices=choices)

	elif type == FormField.SELECT_MULTIPLE_HORIZONTAL:
		field = MultipleChoiceField(widget=SelectMultiple(model_field=data), label=label, choices=choices)

	elif type == FormField.SELECT_MULTIPLE_BUTTONS:
		field = MultipleChoiceField(widget=SelectMultiple(model_field=data), label=label, choices=choices)

	elif type == FormField.SELECT_MULTIPLE_IMAGES:
		field = MultipleChoiceField(widget=SelectMultiple(model_field=data), label=label, choices=choices)

	elif type == FormField.COMMA_SEPARATED_LIST:
		field = forms.CharField(widget=SingleLineText(model_field=data), label=label)

	elif type == FormField.FILE:
		field = forms.FileField(widget=ClearableFile(model_field=data), label=label)

	elif type == FormField.SECURE_FILE:
		field = forms.FileField(widget=ClearableFile(model_field=data), label=label)

	elif type == FormField.DATE:
		field = forms.DateField(widget=SingleLineText(model_field=data), label=label)

	elif type == FormField.TIME:
		field = forms.TimeField(widget=SingleLineText(model_field=data), label=label)

	elif type == FormField.DATE_TIME:
		field = forms.DateTimeField(widget=SingleLineText(model_field=data), label=label)

	elif type == FormField.HIDDEN_FIELD:
		field = forms.CharField(widget=SingleLineText(model_field=data), label=label)

	elif type == FormField.HONEYPOT_FIELD:
		field = forms.CharField(widget=SingleLineText(model_field=data), label=label)
		
	else:
		#DEFAULT:
		field = forms.CharField(widget=SingleLineText(model_field=data), label=label)

	# is_required = models.BooleanField(default=False, 
	#     help_text=help['is_required'])    
	# is_digits = models.BooleanField(default=False, 
	#     help_text=help['is_digits'])
	# is_alphanumeric = models.BooleanField(default=False, 
	#     help_text=help['is_alphanumeric'])

	if field:

		field.required = data.is_required    


		field.secondary_label = '' if hasattr(data, 'secondary_label')==False else data.secondary_label
		field.placeholder_text = '' if hasattr(data, 'placeholder_text')==False else data.placeholder_text
		field.help_text = '' if hasattr(data, 'help_text')==False else data.help_text
		field.field_model = data

	
	return field


from django.conf.urls import patterns, url, include

from .views import *

urlpatterns = patterns('',
	
	# url( (r'^%s$'%settings.BLOG_TAG_DOMAIN), BlogTagListView.as_view(), {'path': "/%s"%(settings.BLOG_TAG_DOMAIN)}, name='blog_tags'),
	# url( (r'^%s$'%settings.BLOG_ARTICLE_DOMAIN), BlogListView.as_view(), {'path': "/%s"%(settings.BLOG_ARTICLE_DOMAIN)}, name='blog_list'),	
	# url( (r'^%s$'%settings.BLOG_CONTRIBUTOR_DOMAIN), BlogContributorListView.as_view(), {'path': "/%s"%(settings.BLOG_CONTRIBUTOR_DOMAIN)}, name='blog_contributors'),	
	# url( (r'^%s(?P<path>[\w-]+)/$'%settings.BLOG_TAG_DOMAIN), BlogTagView.as_view(), name='blog_tag'),
	# url( (r'^%s(?P<path>[\w-]+)/$'%settings.BLOG_CATEGORY_DOMAIN), BlogCategoryView.as_view(), name='blog_category'),
	# url( (r'^%s(?P<path>.*)/$'%settings.BLOG_ARTICLE_DOMAIN), BlogArticleDetailView.as_view(), name='blog_article'),

)


import os
from django.conf import settings
from django.core.management.base import BaseCommand, CommandError

from carbon.utils.template import get_page_templates, get_page_templates_raw

from ...models import Template


class Command(BaseCommand):
    args = ''
    help = ''

    def handle(self, *args, **options):

        print "TODO..."
        


from __future__ import unicode_literals
import urlparse

from django.conf import settings
from django.http import HttpResponsePermanentRedirect
try:
    from django.apps import apps
    get_model = apps.get_model
except:
    from django.db.models.loading import get_model


class LegacyURLMiddleware(object):

    # Defined as class-level attributes to be subclassing-friendly.
    
    def process_response(self, request, response):
        # No need to check for a redirect for non-404 responses.
        if response.status_code != 404:
            return response

        try:
            full_path = request.get_full_path()
            legacy_url_model = get_model(settings.LEGACY_URL_MODEL.split('.')[0], settings.LEGACY_URL_MODEL.split('.')[1])
            all_legacy_urls = legacy_url_model.objects.all()
            legacy_url = legacy_url_model.objects.get(url=full_path)
            return HttpResponsePermanentRedirect(legacy_url.path)
        except:
            pass

        if not full_path.endswith('/'):
            if settings.APPEND_SLASH:

                if '?' in full_path:
                    pieces = full_path.split('?')
                    slashed_path = '%s/%s'%(pieces[0], pieces[1])
                else:
                    slashed_path = '%s/'%full_path
                
                return HttpResponsePermanentRedirect(slashed_path)
                

        return response


from django import forms
from django.forms.widgets import HiddenInput
from django.utils.safestring import mark_safe

from django_ace import AceWidget

from .models import *


from carbon.compounds.form.utils import get_form_field_by_type, FieldData




class EmailTemplateAdminForm(forms.ModelForm):
    body_template = forms.CharField(widget=AceWidget(mode='html', width="850px", height="800px", showprintmargin=True), required=False)
    subject_template = forms.CharField(widget=AceWidget(mode='html', width="850px", height="200px", showprintmargin=True), required=False)
    
class EmailCategorySubscriptionSettingsForm(forms.Form):
	from carbon.compounds.form.models import FormField

	parent = forms.CharField(widget=HiddenInput())

	category = get_form_field_by_type(FormField.HIDDEN_FIELD, FieldData(is_required=True))
	status = get_form_field_by_type(FormField.SELECT_RADIO_BUTTONS, FieldData(is_required=True), 
		choices=EmailCategorySubscriptionSettings.NOTIFICATION_STATUS_CHOICES,
		label='Subscription Status')

	# print 'parent? %s'%(parent)
	# print 'category? %s'%(category)


try:
    from urllib.parse import urlparse
except ImportError:     # Python 2
    from urlparse import urlparse
from functools import wraps
from django.conf import settings
from django.utils.decorators import decorator_from_middleware_with_args, available_attrs
from django.views.decorators.cache import cache_page
from django.contrib.messages.api import get_messages


def user_cache_test(test_func, cache_duration=settings.CACHE_DURATION):
    """
    Decorator for views that checks that the user passes the given test,
    redirecting to the log-in page if necessary. The test should be a callable
    that takes the user object and returns True if the user passes.
    """

    def decorator(view_func):
        @wraps(view_func, assigned=available_attrs(view_func))
        def _wrapped_view(request, *args, **kwargs):

            #Don't cache pages that have unique messages
            messages = get_messages(request)
            if len(messages) > 0 or test_func(request.user):

                return view_func(request, *args, **kwargs)

            else:
                return cache_page(cache_duration)(view_func)(request, *args, **kwargs) 
                

        return _wrapped_view
    return decorator


def admins_skip_cache(function=None, cache_duration=settings.CACHE_DURATION):
    """
    Decorator for views that checks that the user is logged in, redirecting
    to the log-in page if necessary.
    """
    actual_decorator = user_cache_test(
        lambda u: u.is_authenticated() and u.is_staff,
        cache_duration=cache_duration
    )
    if function:
        return actual_decorator(function)
    return actual_decorator

def users_skip_cache(function=None, cache_duration=settings.CACHE_DURATION):
    """
    Decorator for views that checks that the user is logged in, redirecting
    to the log-in page if necessary.
    """
    actual_decorator = user_cache_test(
        lambda u: u.is_authenticated(),
        cache_duration=cache_duration
    )
    if function:
        return actual_decorator(function)
    return actual_decorator    


import sys, traceback

from django.template import Library
from django.core.urlresolvers import reverse
from django.conf import settings
from django.utils.safestring import mark_safe
try:
    from django.apps import apps
    get_model = apps.get_model
except:
    from django.db.models.loading import get_model

register = Library()


@register.assignment_tag(takes_context=True)
def get_rendered_field(context, field):
    
    if hasattr(field.field.widget, 'has_custom_render') and field.field.widget.has_custom_render==True:

        try:
            rendered = field.field.widget.render(field, context)            
        except Exception, err:  
            rendered = traceback.print_exc()            

        return rendered
    else:
        return field.field.widget.render(field.html_name, field.value())

@register.filter
def tostring(value):
    return str(value)        


from django.conf import settings
from django.contrib import messages
from django.core.exceptions import ImproperlyConfigured
from django.core.urlresolvers import reverse
import django.dispatch
from django.http import QueryDict

from django.views.generic.detail import DetailView
from django.views.generic.edit import FormMixin, ProcessFormView
from django.template import loader, Context, RequestContext, Template
from django.template.response import SimpleTemplateResponse

from carbon.atoms.views.abstract import *
from carbon.atoms.views.content import *


signal_form_error = django.dispatch.Signal(providing_args=["form_schema"])
signal_form_entry_created = django.dispatch.Signal(providing_args=["form_schema", "form_entry"])
signal_form_entry_updated = django.dispatch.Signal(providing_args=["form_schema", "form_entry"])


class FormSignalOperator(FormMixin, ProcessFormView):
    default_error_message = "Please correct the errors below."
    default_success_message = "Your form was saved."

    def get_form_schema(self):
        raise ImproperlyConfigured("View should override get_form_schema() function")

    def get_form_schema(self):
        raise ImproperlyConfigured("View should override get_form_schema() function")

    def get_is_create_form(self):
        raise ImproperlyConfigured("View should override get_is_create_form() function")

    def form_invalid(self, form):
        form_schema = self.get_form_schema()
        if form_schema:
            messages.error(self.request, form_schema.form_error_message or self.default_error_message)
            form_schema_class = type(form_schema)
            signal_form_error.send(sender=form_schema_class, form_schema=form_schema)
        else:
            messages.error(self.request, self.default_error_message)

        return self.render_to_response(self.get_context_data(form=form))


    def form_valid(self, form):
        self.form_entry_object = form.save()
        created = self.get_is_create_form()

        form_schema = self.get_form_schema()
        if form_schema:
            messages.success(self.request, form_schema.form_create_message or self.default_success_message)        
            form_schema.handle_successful_submission(form, self.form_entry_object, created)

            form_entry_class = type(self.form_entry_object)
            if created:
                signal_form_entry_created.send(sender=form_entry_class, form_schema=form_schema, form_entry=self.form_entry_object)
            else:
                signal_form_entry_updated.send(sender=form_entry_class, form_schema=form_schema, form_entry=self.form_entry_object)

        return super(FormSignalOperator, self).form_valid(form)



class FormSubmittedView(AddressibleView, PublishableView, DetailView):
    
    def get_template_names(self):

        return [self.object.submit_template.slug]

    

class CreateFormEntryMixin(FormSignalOperator):
    content_type = None
    form_entry_object = None
    default_success_message = "Your form was created."

    def get_form_schema(self):
        #Override in subclass
        raise ImproperlyConfigured("View should override get_form_schema() function")

    def get_is_create_form(self):
        return True

    def get(self, request, *args, **kwargs):
        self.form_schema = self.get_form_schema()
        if self.form_schema:
            self.form = form = self.get_form()
        return super(CreateFormEntryMixin, self).get(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        self.form_schema = self.get_form_schema()
        if self.form_schema:
            self.form = form = self.get_form()
        
        return super(CreateFormEntryMixin, self).post(request, *args, **kwargs)
    

    def get_form_kwargs(self):      
        #ADD REFERNCE TO FORM OBJECT
        kwargs = super(CreateFormEntryMixin, self).get_form_kwargs()
        kwargs['form_schema'] = self.form_schema
        kwargs['request'] = self.request
        return kwargs

    def get_success_url(self):
        return self.form_schema.get_success_url(self.form_entry_object)


    def get_context_data(self, **kwargs):
        context = super(CreateFormEntryMixin, self).get_context_data(**kwargs)
        context['form'] = None
        context['has_form'] = False
        context['form_entry_object'] = self.form_entry_object
        context['form_schema'] = self.form_schema

        if hasattr(self, 'form'):
            context['has_form'] = True
            context['form'] = self.form
        return context


class CreateFormEntryView(AddressibleView, PublishableView, FormSignalOperator):
    content_type = None
    form_entry_object = None
    default_success_message = "Your form was created."

    def get_form_kwargs(self):
        
        #ADD REFERNCE TO FORM OBJECT
        kwargs = super(CreateFormEntryView, self).get_form_kwargs()
        kwargs['form_schema'] = self.object
        kwargs['request'] = self.request
        return kwargs

    def get_success_url(self):
        return self.object.get_success_url(self.form_entry_object)

    def get_form_schema(self):
        return self.object

    def get_is_create_form(self):
        return True


    def get_context_data(self, **kwargs):
        context = super(CreateFormEntryView, self).get_context_data(**kwargs)
        context['form_entry_object'] = self.form_entry_object
        return context


class UpdateFormEntryView(PublishableView, AddressibleView, FormSignalOperator):
    content_type = None
    form_entry_object = None
    form_entry_class = None
    default_success_message = "Your form was updated."

    def get_form_kwargs(self):
        
        kwargs = super(UpdateFormEntryView, self).get_form_kwargs()

        #ADD REFERNCE TO FORM OBJECT AND FIELD DATA
        kwargs['form_schema'] = self.object
        kwargs['request'] = self.request
        kwargs['instance'] = self.form_entry_object
        

        if self.request.method in ('POST', 'PUT'):
            
            kwargs.update({
                'data': self.request.POST,
                'files': self.request.FILES,
            })
            
        else:
            
            data = {'form_schema':self.object.pk}
            fields = self.form_entry_object.get_entries()
            for field in fields:
                key = self.form_class.form_field_prefix+field.form_field.slug
                data[key] = field.decompressed_value

            qdict = QueryDict('', mutable=True)
            qdict.update(data)
            kwargs['data'] = qdict

        

        return kwargs

    def get(self, request, *args, **kwargs):
        self.form_entry_object = self.get_form_entry()
        return super(UpdateFormEntryView, self).get(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        self.form_entry_object = self.get_form_entry()
        return super(UpdateFormEntryView, self).post(request, *args, **kwargs)

    def get_form_entry(self):
        if self.form_entry_object:
            return self.form_entry_object

        pk = self.kwargs.get('pk', '')

        queryset = self.form_entry_class._default_manager.all()
        try:
            obj = queryset.filter(pk=pk)[0]
        except:
            raise Http404(_("No %(verbose_name)s found matching the query") %
                    {'verbose_name': queryset.model._meta.verbose_name})
        return obj


    def get_success_url(self):
        return self.object.get_success_url(self.form_entry_object)

    def get_form_schema(self):
        return self.form_entry_object.form_schema

    def get_is_create_form(self):
        return False


    def get_context_data(self, **kwargs):
        context = super(UpdateFormEntryView, self).get_context_data(**kwargs)
        context['form_entry_object'] = self.form_entry_object
        return context


from django.template import Library
from django.core.urlresolvers import reverse
try:
    from django.apps import apps
    get_model = apps.get_model
except:
    from django.db.models.loading import get_model

register = Library()


@register.assignment_tag()
def get_variant_url(object, variant_name):
    return object.get_variant_url(variant_name)

@register.assignment_tag()
def get_variant_width(object, variant_name):
    return object.get_variant_width(variant_name)

@register.assignment_tag()
def get_variant_height(object, variant_name):
    return object.get_variant_height(variant_name)


@register.filter
def filename(full_path):
    head, tail = os.path.split(full_path)
    return tail

@register.assignment_tag()
def get_variant_links(object):
  
  output = [object.get_variant_link('image')]  
  for variant in object.variants:
    output.append(object.get_variant_link(variant))
  return output    

@register.assignment_tag()
def get_secure_link(object, duration):
    return object.get_secure_url(duration)    


@register.simple_tag
def upload_js():
    return """
<!-- The template to display files available for upload -->
<script id="template-upload" type="text/x-tmpl">
{% for (var i=0, file; file=o.files[i]; i++) { %}
    <tr class="template-upload fade">
        <td>
            <span class="preview"></span>
        </td>
        <td>
            <p class="name">{%=file.name%}</p>
            {% if (file.error) { %}
                <div><span class="label label-important">{%=locale.fileupload.error%}</span> {%=file.error%}</div>
            {% } %}
        </td>
        <td>
            <p class="size">{%=o.formatFileSize(file.size)%}</p>
            {% if (!o.files.error) { %}
                <div class="progress progress-striped active" role="progressbar" aria-valuemin="0" aria-valuemax="100" aria-valuenow="0"><div class="progress-bar progress-bar-success" style="width:0%;"></div></div>
            {% } %}
        </td>
        <td>
            {% if (!o.files.error && !i && !o.options.autoUpload) { %}
                <button class="btn btn-primary start">
                    <i class="glyphicon glyphicon-upload"></i>
                    <span>{%=locale.fileupload.start%}</span>
                </button>
            {% } %}
            {% if (!i) { %}
                <button class="btn btn-warning cancel">
                    <i class="glyphicon glyphicon-ban-circle"></i>
                    <span>{%=locale.fileupload.cancel%}</span>
                </button>
            {% } %}
        </td>
    </tr>
{% } %}
</script>
<!-- The template to display files available for download -->
<script id="template-download" type="text/x-tmpl">
{% for (var i=0, file; file=o.files[i]; i++) { %}
    <tr class="template-download fade">
        <td>
            <span class="preview">
                {% if (file.thumbnailUrl) { %}
                    <a href="{%=file.url%}" title="{%=file.name%}" download="{%=file.name%}" data-gallery><img src="{%=file.thumbnailUrl%}"></a>
                {% } %}
            </span>
        </td>
        <td>
            <p class="name">
                <a href="{%=file.url%}" title="{%=file.name%}" download="{%=file.name%}" {%=file.thumbnailUrl?'data-gallery':''%}>{%=file.name%}</a>
            </p>
            {% if (file.error) { %}
                <div><span class="label label-important">{%=locale.fileupload.error%}</span> {%=file.error%}</div>
            {% } %}
        </td>
        <td>
            <span class="size">{%=o.formatFileSize(file.size)%}</span>
        </td>
        <td>
            <button class="btn btn-danger delete" data-type="{%=file.deleteType%}" data-url="{%=file.deleteUrl%}"{% if (file.deleteWithCredentials) { %} data-xhr-fields='{"withCredentials":true}'{% } %}>
                <i class="glyphicon glyphicon-trash"></i>
                <span>{%=locale.fileupload.destroy%}</span>
            </button>
            <input type="checkbox" name="delete" value="1" class="toggle">
        </td>
    </tr>
{% } %}
</script>
"""
    


#!/usr/bin/env python
#CODE HELP FROM https://code.google.com/p/threepress/source/browse/trunk/bookworm/database-backup.py


import sys
import os.path
import os
import logging
from datetime import datetime
from django.conf import settings
import boto.s3

logging.basicConfig(level=logging.INFO, stream=sys.stdout)

MYSQL_CMD = 'mysqldump'
ZIP_CMD = 'zip'

def _setup(backup_dir):
    if not os.path.exists(backup_dir):
        logging.debug("Created backup directory %s" % backup_dir)
        os.mkdir(backup_dir)
    else:
        logging.debug("Using backup directory %s" % backup_dir)
    
def _backup_name(prefix="database_backup"):

    now = datetime.now()
    now_string ="%s-%s-%s_%s-%s" % (now.year, str(now.month).zfill(2), str(now.day).zfill(2), str(now.hour).zfill(2), str(now.minute).zfill(2))
    file_name = "%s_%s.sql" % (prefix, now_string)

    logging.debug("Setting backup name for as %s" % (file_name))
    return file_name

def _run_backup(file_name, backup_dir, database_settings="default"):
    user = settings.DATABASES[database_settings]['USER']
    password = settings.DATABASES[database_settings]['PASSWORD']
    name = settings.DATABASES[database_settings]['NAME']
    port = '3306'#settings.DATABASES[database_settings]['PORT']
    host = settings.DATABASES[database_settings]['HOST']
    
    #mysqldump -P 3310 -h 127.0.0.1 -u mysql_user -p database_name table_name

    cmd = "%(mysqldump)s -P %(port)s -h %(host)s -u %(user)s --password=%(password)s %(database)s > %(log_dir)s/%(file)s" % {
        'mysqldump' : MYSQL_CMD,
        'port' : port,
        'host' : host,
        'user' : user,
        'password' : password,
        'database' : name,
        'log_dir' : backup_dir,
        'file': file_name}
    logging.debug("Backing up with command %s " % cmd)
    return os.system(cmd)

def _zip_backup(file_name, backup_dir):
    backup = "%s/%s" % (backup_dir, file_name)
    zipfile_name = "%s.zip" % (backup)

    if os.path.exists(zipfile_name):
        logging.debug("Removing previous zip archive %s" % zipfile_name)
        os.remove(zipfile_name)
    zip_cmds = {'zip' : ZIP_CMD, 'zipfile' : zipfile_name, 'file' : backup }

    # Create the backup
    logging.debug("Making backup as %s " % zipfile_name)
    os.system("%(zip)s -q -9 %(zipfile)s %(file)s" % zip_cmds)

    # Test our archive
    logging.debug("Testing zip archive")
    if not os.system("%(zip)s -T -D -q %(zipfile)s" % zip_cmds):
        # If there was no problem, then delete the unzipped version
        os.remove(backup)
        return zipfile_name
    else:
        return None

def _upload_backup(bucket_name, file_name, dest_dir):
    base_name = os.path.basename(file_name)
        
    conn = boto.connect_s3(settings.AWS_ACCESS_KEY_ID, settings.AWS_SECRET_ACCESS_KEY)
    bucket = conn.get_bucket(bucket_name)

    print 'Uploading %s to Amazon S3 from %s' % (base_name, file_name)

    import sys
    def percent_cb(complete, total):
        sys.stdout.write('.')
        sys.stdout.flush()

    from boto.s3.key import Key
    k = Key(bucket)
    k.key = u"%s%s"%(dest_dir, base_name)
    k.set_contents_from_filename(file_name, cb=percent_cb, num_cb=10)

    #Remove tmp file
    os.remove(file_name)

def backup_database(bucket_name, database_settings="default", dest_dir="database/", prefix="database_backup", tmp_dir="/tmp"):
    _setup(tmp_dir)
    file_name = _backup_name(prefix)
    
    _run_backup(file_name, tmp_dir, database_settings)
    zipped_file = _zip_backup(file_name, tmp_dir)

    _upload_backup(bucket_name, zipped_file, dest_dir)
        
    
if __name__ == '__main__':
    sys.exit(main(*sys.argv))  


from django import forms
from django_ace import AceWidget
from ace_overlay.widgets import AceOverlayWidget
from django.utils.safestring import mark_safe
from .models import *


class TemplateAdminForm(forms.ModelForm):
    custom_template = forms.CharField(widget=AceOverlayWidget(mode='html', wordwrap=True, theme='github', width="850px", height="700px", showprintmargin=True), required=False)
    # class Meta:
    #     model = Template


class CSSResourceAdminForm(forms.ModelForm):
    custom_source = forms.CharField(widget=AceOverlayWidget(mode='sass', wordwrap=True, theme='github', width="850px", height="800px", showprintmargin=True), required=False)
    # class Meta:
    #     model = CSSResource   

class JSResourceAdminForm(forms.ModelForm):
    custom_source = forms.CharField(widget=AceOverlayWidget(mode='javascript', wordwrap=True, theme='github', width="850px", height="800px", showprintmargin=True), required=False)
    # class Meta:
    #     model = JSResource   


class UploadFileForm(forms.Form):
    file  = forms.FileField(required=True)
        
class MenuItemForm(forms.ModelForm):
    
    def __init__(self, *args, **kwargs):
        super(MenuItemForm, self).__init__(*args, **kwargs)
        self.fields['content_type'].help_text = mark_safe('To set this link to an existing\
        item in the cms, choose a Content Type and Object ID.')
        
        self.fields['path_override'].help_text = mark_safe('To set this link to an external \
        URL or absolute path, use the Path Override field.<br /><br />\
        External URLs should use this format "http://www.example.com" and absolute \
        URLs should use this format "/path/to/page/"') 


from django.contrib import messages
from django.http import Http404
from django.shortcuts import redirect, render_to_response
from django.template import RequestContext
from django.utils.translation import ugettext as _
from django.views.generic import DetailView
from django.views.generic.base import RedirectView

try:
    from django.apps import apps
    get_model = apps.get_model
except:
    from django.db.models.loading import get_model

from carbon.atoms.views.abstract import *
from carbon.atoms.views.content import *

from .models import *
from .forms import *


def admin_import_links( request ):    
    

    if not request.user or not request.user.is_staff:
        raise Http404()

    # Handle file upload
    if request.method == 'POST':
        form = UploadFileForm(request.POST, request.FILES)
        import_file = request.FILES.get('file', None)

        if import_file:

            app_label = settings.LEGACY_URL_MODEL.split('.')[0]
            object_name = settings.LEGACY_URL_MODEL.split('.')[1]
            model = get_model(app_label, object_name)
            
            results = model.import_links(import_file, request)

            messages.success(request, results)

        else:
            messages.warning(request, 'No .CSV file specified')
       
    else:
        form = UploadFileForm()
        pass

    

    # Render list page with the documents and the form
    return render_to_response(
        'admin/page/import_legacy_urls.html',
        {'form': form},
        context_instance=RequestContext(request)
    )    


from django.utils.safestring import mark_safe
from django.contrib import admin
from django.contrib.auth.admin import UserAdmin as ContribUserAdmin
from django.utils.translation import ugettext_lazy as _

from .forms import *
# from .models import User, Organization

from reversion.admin import VersionAdmin
from django_inline_wrestler.admin import TabularInlineOrderable

from carbon.atoms.admin.content import BaseVersionableAdmin, BaseVersionableTitleAdmin



class UserAdmin(ContribUserAdmin):
    def preview(self, obj):
        if obj.image:
            try:
                return mark_safe("<img src='%s' alt='%s preview'/>"%(obj.image.thumbnail_url, obj.image.title))
            except:
                return ""
        return ''
    

    autocomplete_lookup_fields = {
        'fk': ['image',],
    }
    raw_id_fields = ('image',)

    form = UserChangeForm
    add_form = UserCreationForm
    add_fieldsets = (
        ("User", {
            'classes': (),
            'fields': ('email', 'password1', 'password2','first_name','last_name',)}
        ),
    )

    fieldsets = (
        ('User', { 
            'fields': (
                ("first_name","last_name"),
                ('email','date_of_birth'),
                'password',
                'about',
                ('preview','image')
            ),
            'classes': ( 'grp-collapse grp-open', )
        }),
        (_('CMS Permissions'), {
            'fields': (
                ('is_active', 'is_staff',),
                
                'is_superuser',
                ('groups')
            ),
            'classes': ( 'grp-collapse grp-closed', )
        }),
    )

    list_display = ('preview','email', 'first_name', 'last_name', 'is_staff', 'is_superuser')
    list_display_links = ('preview','email','first_name','last_name')
    list_filter = ('is_staff', 'is_superuser', 'is_active', 'groups')
    search_fields = ('first_name', 'last_name', 'email')
    ordering = ('email',)
    readonly_fields = ContribUserAdmin.readonly_fields + ('preview',)
    #'last_login','date_joined', 'impersonate_user'




class UserGroupAdmin(VersionAdmin, BaseVersionableTitleAdmin):
    list_display = ('title', 'order')
    list_editable = ('order',)

    form = UserGroupAdminForm

    core_fields = (
        ('title','slug'),
        ('synopsis'),
        ('content')
    )
    meta_fields = BaseVersionableAdmin.meta_fields
    fieldsets = (
        ("Main Body", {
            'fields': core_fields,
            'classes': ( 'grp-collapse grp-open', )
        }),
        
        ("Meta", {
            'fields': meta_fields,
            'classes': ( 'grp-collapse grp-closed', )
        })
    )
    search_fields = ('title','admin_note', 'synopsis', 'content')


class UserGroupMemberInGroupAdmin(TabularInlineOrderable):
    #model = UserGroupMember

    def edit_member_link(self, obj):
        if obj.user:
            try:
                return mark_safe("<a href='%s'>Edit User></a>"%(obj.user.edit_item_url))
            except:
                return ""
        return ''

    fk_name = 'group'
    autocomplete_lookup_fields = {
        'fk': ['user',],
    }
    raw_id_fields = ('user',)
    fields = ('order','user','edit_member_link')
    extra = 0
    readonly_fields = ('edit_member_link',)

class UserGroupMemberInUserAdmin(admin.TabularInline):
    #model = UserGroupMember

    def all_members_link(self, obj):
        if obj.group:
            try:
                return mark_safe("<a href='%s'>See All Members ></a>"%(obj.group.edit_item_url))
            except:
                return ""
        return ''

    fk_name = 'user'
    autocomplete_lookup_fields = {
        'fk': ['group',],
    }
    raw_id_fields = ('group',)
    fields = ('group','all_members_link',)
    extra = 0    
    readonly_fields = ('all_members_link',)


class OrganizationAdmin(VersionAdmin, BaseVersionableTitleAdmin):
    list_display = ('title', 'order')
    list_editable = ('order',)

    core_fields = (
        ('title','slug'),
        ('synopsis'),
        ('content')
    )
    meta_fields = BaseVersionableAdmin.meta_fields
    fieldsets = (
        ("Main Body", {
            'fields': core_fields,
            'classes': ( 'grp-collapse grp-open', )
        }),
        
        ("Meta", {
            'fields': meta_fields,
            'classes': ( 'grp-collapse grp-closed', )
        })
    )
    search_fields = ('title','admin_note', 'synopsis', 'content')
    
    

class SocialContactLinkInline(TabularInlineOrderable):
    # model = SocialContactLink
    fk_name = 'user'    
    fields = ('order','title','url','icon', 'css_classes', 'extra_attributes')
    extra = 0

# admin.site.register(Organization, OrganizationAdmin)    
# admin.site.register(User, UserAdmin)





from django.utils import timezone
from django.contrib import messages

from ..data.csv import ModelToCSV

#ADMIN VIEWS 



def resave_item(modeladmin, request, queryset):	

	items_saved = 0
	save_errors = 0
	for item in queryset:
	    try:
	    	item.save()
	    	items_saved += 1
	    except:
	    	save_errors += 1

	if items_saved > 0:
	 	messages.success(request, '%s items were saved.'%(items_saved))

	if save_errors > 0:
		messages.warning(request, '%s items were not saved.'%(save_errors))
resave_item.short_description = "Re-Save"


def output_csv(modeladmin, request, queryset):
	
	try:

		verbose_name = modeladmin.model._meta.verbose_name_plural.title()
		now = timezone.now()
		now_string ="%s-%s-%s" % (now.year, str(now.month).zfill(2), str(now.day).zfill(2))
		filename = "%s_%s.csv" % (verbose_name, now_string)

		return ModelToCSV(request, queryset, modeladmin.csv_fields, filename=filename)

	except:
		messages.warning(request, 'There are no CSV fields specified for this model')
output_csv.short_description = "Output to CSV"    


from django.conf.urls.static import static
from django.conf.urls import include, url

from django.views.generic import TemplateView

from django.contrib.admindocs import views

from .views import *
from .views_admindocs import *

urlpatterns = [
    
    
    url(r'^admin/utils/cache/clear/', clear_cache, name="clear-server-cache"),
    url(r'^admin/grappelli/lookup/autocomplete/$', FilteredAutocomplete.as_view(), name="grp_autocomplete_lookup"),
    url('^admin/doc/models/(?P<app_label>[^\.]+)\.(?P<model_name>[^/]+)/$',
        custom_model_detail,
        name='django-admindocs-models-detail'
    ),

    url(r'^error_403/', TemplateView.as_view(template_name="403.html"), name="error_403_preview" ),
    url(r'^error_404/', TemplateView.as_view(template_name="404.html"), name="error_404_preview" ),
    url(r'^error_405/', TemplateView.as_view(template_name="405.html"), name="error_405_preview" ),
    url(r'^error_500/', TemplateView.as_view(template_name="500.html"), name="error_500_preview" ),
    url(r'^error_501/', TemplateView.as_view(template_name="501.html"), name="error_501_preview" ),

    url(r'^error_maintenance/', TemplateView.as_view(template_name="maintenance.html"), name="error_maintenance_preview" ),    
    
]


#Font Awesome 4.3.0

ICON_CHOICES = (
	('glass','Glass'),
	('music', 'Music'),
	('search', 'Search'),
	('envelope-o', 'Envelope O'),
	('heart', 'Heart'),
	('star', 'Star'),
	('star-o', 'Star o'),
	('user', 'User'),
	('film', 'Film'),
	('th-large', 'Th Large'),
	('th', 'Th'),
	('th-list', 'Th List'),
	('check', 'Check'),
	('remove', 'Remove'),
	('close', 'Close'),
	('times', 'Times'),
	('search-plus', 'Search Plus'),
	('search-minus', 'Search Minus'),
	('power-off', 'Power Off'),
	('signal', 'Signal'),
	('gear', 'Gear'),
	('cog', 'Cog'),
	('trash-o', 'Trash o'),
	('home', 'Home'),
	('file-o', 'File o'),
	('clock-o', 'Clock o'),
	('road', 'Road'),
	('download', 'Download'),
	('arrow-circle-o-down', 'Arrow circle-o Down'),
	('arrow-circle-o-up', 'Arrow circle-o Up'),
	('inbox', 'Inbox'),
	('play-circle-o', 'Play circle o'),
	('rotate-right', 'Rotate Right'),
	('repeat', 'Repeat'),
	('refresh', 'Refresh'),
	('list-alt', 'List Alt'),
	('lock', 'Lock'),
	('flag', 'Flag'),
	('headphones', 'Headphones'),
	('volume-off', 'Volume Off'),
	('volume-down', 'Volume Down'),
	('volume-up', 'Volume Up'),
	('qrcode', 'Qrcode'),
	('barcode', 'Barcode'),
	('tag', 'Tag'),
	('tags', 'Tags'),
	('book', 'Book'),
	('bookmark', 'Bookmark'),
	('print', 'Print'),
	('camera', 'Camera'),
	('font', 'Font'),
	('bold', 'Bold'),
	('italic', 'Italic'),
	('text-height', 'Text Height'),
	('text-width', 'Text Width'),
	('align-left', 'Align Left'),
	('align-center', 'Align Center'),
	('align-right', 'Align Right'),
	('align-justify', 'Align Justify'),
	('list', 'List'),
	('dedent', 'Dedent'),
	('outdent', 'Outdent'),
	('indent', 'Indent'),
	('video-camera', 'Video Camera'),
	('photo', 'Photo'),
	('image', 'Image'),
	('picture-o', 'Picture o'),
	('pencil', 'Pencil'),
	('map-marker', 'Map Marker'),
	('adjust', 'Adjust'),
	('tint', 'Tint'),
	('edit', 'Edit'),
	('pencil-square-o', 'Pencil square o'),
	('share-square-o', 'Share square o'),
	('check-square-o', 'Check square o'),
	('arrows', 'Arrows'),
	('step-backward', 'Step Backward'),
	('fast-backward', 'Fast Backward'),
	('backward', 'Backward'),
	('play', 'Play'),
	('pause', 'Pause'),
	('stop', 'Stop'),
	('forward', 'Forward'),
	('fast-forward', 'Fast Forward'),
	('step-forward', 'Step Forward'),
	('eject', 'Eject'),
	('chevron-left', 'Chevron Left'),
	('chevron-right', 'Chevron Right'),
	('plus-circle', 'Plus Circle'),
	('minus-circle', 'Minus Circle'),
	('times-circle', 'Times Circle'),
	('check-circle', 'Check Circle'),
	('question-circle', 'Question Circle'),
	('info-circle', 'Info Circle'),
	('crosshairs', 'Crosshairs'),
	('times-circle-o', 'Times circle o'),
	('check-circle-o', 'Check circle o'),
	('ban', 'Ban'),
	('arrow-left', 'Arrow Left'),
	('arrow-right', 'Arrow Right'),
	('arrow-up', 'Arrow Up'),
	('arrow-down', 'Arrow Down'),
	('mail-forward', 'Mail Forward'),
	('share', 'Share'),
	('expand', 'Expand'),
	('compress', 'Compress'),
	('plus', 'Plus'),
	('minus', 'Minus'),
	('asterisk', 'Asterisk'),
	('exclamation-circle', 'Exclamation Circle'),
	('gift', 'Gift'),
	('leaf', 'Leaf'),
	('fire', 'Fire'),
	('eye', 'Eye'),
	('eye-slash', 'Eye Slash'),
	('warning', 'Warning'),
	('exclamation-triangle', 'Exclamation Triangle'),
	('plane', 'Plane'),
	('calendar', 'Calendar'),
	('random', 'Random'),
	('comment', 'Comment'),
	('magnet', 'Magnet'),
	('chevron-up', 'Chevron Up'),
	('chevron-down', 'Chevron Down'),
	('retweet', 'Retweet'),
	('shopping-cart', 'Shopping Cart'),
	('folder', 'Folder'),
	('folder-open', 'Folder Open'),
	('arrows-v', 'Arrows v'),
	('arrows-h', 'Arrows h'),
	('bar-chart-o', 'Bar chart o'),
	('bar-chart', 'Bar Chart'),
	('twitter-square', 'Twitter Square'),
	('facebook-square', 'Facebook Square'),
	('camera-retro', 'Camera Retro'),
	('key', 'Key'),
	('gears', 'Gears'),
	('cogs', 'Cogs'),
	('comments', 'Comments'),
	('thumbs-o-up', 'Thumbs o Up'),
	('thumbs-o-down', 'Thumbs o Down'),
	('star-half', 'Star Half'),
	('heart-o', 'Heart o'),
	('sign-out', 'Sign Out'),
	('linkedin-square', 'Linkedin Square'),
	('thumb-tack', 'Thumb Tack'),
	('external-link', 'External Link'),
	('sign-in', 'Sign In'),
	('trophy', 'Trophy'),
	('github-square', 'Github Square'),
	('upload', 'Upload'),
	('lemon-o', 'Lemon o'),
	('phone', 'Phone'),
	('square-o', 'Square o'),
	('bookmark-o', 'Bookmark o'),
	('phone-square', 'Phone Square'),
	('twitter', 'Twitter'),
	('facebook-f', 'Facebook f'),
	('facebook', 'Facebook'),
	('github', 'Github'),
	('unlock', 'Unlock'),
	('credit-card', 'Credit Card'),
	('rss', 'Rss'),
	('hdd-o', 'Hdd o'),
	('bullhorn', 'Bullhorn'),
	('bell', 'Bell'),
	('certificate', 'Certificate'),
	('hand-o-right', 'Hand o Right'),
	('hand-o-left', 'Hand o Left'),
	('hand-o-up', 'Hand o Up'),
	('hand-o-down', 'Hand o Down'),
	('arrow-circle-left', 'Arrow circle Left'),
	('arrow-circle-right', 'Arrow circle Right'),
	('arrow-circle-up', 'Arrow circle Up'),
	('arrow-circle-down', 'Arrow circle Down'),
	('globe', 'Globe'),
	('wrench', 'Wrench'),
	('tasks', 'Tasks'),
	('filter', 'Filter'),
	('briefcase', 'Briefcase'),
	('arrows-alt', 'Arrows Alt'),
	('group', 'Group'),
	('users', 'Users'),
	('chain', 'Chain'),
	('link', 'Link'),
	('cloud', 'Cloud'),
	('flask', 'Flask'),
	('cut', 'Cut'),
	('scissors', 'Scissors'),
	('copy', 'Copy'),
	('files-o', 'Files o'),
	('paperclip', 'Paperclip'),
	('save', 'Save'),
	('floppy-o', 'Floppy o'),
	('square', 'Square'),
	('navicon', 'Navicon'),
	('reorder', 'Reorder'),
	('bars', 'Bars'),
	('list-ul', 'List Ul'),
	('list-ol', 'List Ol'),
	('strikethrough', 'Strikethrough'),
	('underline', 'Underline'),
	('table', 'Table'),
	('magic', 'Magic'),
	('truck', 'Truck'),
	('pinterest', 'Pinterest'),
	('pinterest-square', 'Pinterest Square'),
	('google-plus-square', 'Google plus Square'),
	('google-plus', 'Google Plus'),
	('money', 'Money'),
	('caret-down', 'Caret Down'),
	('caret-up', 'Caret Up'),
	('caret-left', 'Caret Left'),
	('caret-right', 'Caret Right'),
	('columns', 'Columns'),
	('unsorted', 'Unsorted'),
	('sort', 'Sort'),
	('sort-down', 'Sort Down'),
	('sort-desc', 'Sort Desc'),
	('sort-up', 'Sort Up'),
	('sort-asc', 'Sort Asc'),
	('envelope', 'Envelope'),
	('linkedin', 'Linkedin'),
	('rotate-left', 'Rotate Left'),
	('undo', 'Undo'),
	('legal', 'Legal'),
	('gavel', 'Gavel'),
	('dashboard', 'Dashboard'),
	('tachometer', 'Tachometer'),
	('comment-o', 'Comment o'),
	('comments-o', 'Comments o'),
	('flash', 'Flash'),
	('bolt', 'Bolt'),
	('sitemap', 'Sitemap'),
	('umbrella', 'Umbrella'),
	('paste', 'Paste'),
	('clipboard', 'Clipboard'),
	('lightbulb-o', 'Lightbulb o'),
	('exchange', 'Exchange'),
	('cloud-download', 'Cloud Download'),
	('cloud-upload', 'Cloud Upload'),
	('user-md', 'User Md'),
	('stethoscope', 'Stethoscope'),
	('suitcase', 'Suitcase'),
	('bell-o', 'Bell o'),
	('coffee', 'Coffee'),
	('cutlery', 'Cutlery'),
	('file-text-o', 'File text o'),
	('building-o', 'Building o'),
	('hospital-o', 'Hospital o'),
	('ambulance', 'Ambulance'),
	('medkit', 'Medkit'),
	('fighter-jet', 'Fighter Jet'),
	('beer', 'Beer'),
	('h-square', 'h Square'),
	('plus-square', 'Plus Square'),
	('angle-double-left', 'Angle double Left'),
	('angle-double-right', 'Angle double Right'),
	('angle-double-up', 'Angle double Up'),
	('angle-double-down', 'Angle double Down'),
	('angle-left', 'Angle Left'),
	('angle-right', 'Angle Right'),
	('angle-up', 'Angle Up'),
	('angle-down', 'Angle Down'),
	('desktop', 'Desktop'),
	('laptop', 'Laptop'),
	('tablet', 'Tablet'),
	('mobile-phone', 'Mobile Phone'),
	('mobile', 'Mobile'),
	('circle-o', 'Circle o'),
	('quote-left', 'Quote Left'),
	('quote-right', 'Quote Right'),
	('spinner', 'Spinner'),
	('circle', 'Circle'),
	('mail-reply', 'Mail Reply'),
	('reply', 'Reply'),
	('github-alt', 'Github Alt'),
	('folder-o', 'Folder o'),
	('folder-open-o', 'Folder open o'),
	('smile-o', 'Smile o'),
	('frown-o', 'Frown o'),
	('meh-o', 'Meh o'),
	('gamepad', 'Gamepad'),
	('keyboard-o', 'Keyboard o'),
	('flag-o', 'Flag o'),
	('flag-checkered', 'Flag Checkered'),
	('terminal', 'Terminal'),
	('code', 'Code'),
	('mail-reply-all', 'Mail reply All'),
	('reply-all', 'Reply All'),
	('star-half-empty', 'Star half Empty'),
	('star-half-full', 'Star half Full'),
	('star-half-o', 'Star half o'),
	('location-arrow', 'Location Arrow'),
	('crop', 'Crop'),
	('code-fork', 'Code Fork'),
	('unlink', 'Unlink'),
	('chain-broken', 'Chain Broken'),
	('question', 'Question'),
	('info', 'Info'),
	('exclamation', 'Exclamation'),
	('superscript', 'Superscript'),
	('subscript', 'Subscript'),
	('eraser', 'Eraser'),
	('puzzle-piece', 'Puzzle Piece'),
	('microphone', 'Microphone'),
	('microphone-slash', 'Microphone Slash'),
	('shield', 'Shield'),
	('calendar-o', 'Calendar o'),
	('fire-extinguisher', 'Fire Extinguisher'),
	('rocket', 'Rocket'),
	('maxcdn', 'Maxcdn'),
	('chevron-circle-left', 'Chevron circle Left'),
	('chevron-circle-right', 'Chevron circle Right'),
	('chevron-circle-up', 'Chevron circle Up'),
	('chevron-circle-down', 'Chevron circle Down'),
	('html5', 'Html5'),
	('css3', 'Css3'),
	('anchor', 'Anchor'),
	('unlock-alt', 'Unlock Alt'),
	('bullseye', 'Bullseye'),
	('ellipsis-h', 'Ellipsis h'),
	('ellipsis-v', 'Ellipsis v'),
	('rss-square', 'Rss Square'),
	('play-circle', 'Play Circle'),
	('ticket', 'Ticket'),
	('minus-square', 'Minus Square'),
	('minus-square-o', 'Minus square o'),
	('level-up', 'Level Up'),
	('level-down', 'Level Down'),
	('check-square', 'Check Square'),
	('pencil-square', 'Pencil Square'),
	('external-link-square', 'External link Square'),
	('share-square', 'Share Square'),
	('compass', 'Compass'),
	('toggle-down', 'Toggle Down'),
	('caret-square-o-down', 'Caret square-o Down'),
	('toggle-up', 'Toggle Up'),
	('caret-square-o-up', 'Caret square-o Up'),
	('toggle-right', 'Toggle Right'),
	('caret-square-o-right', 'Caret square-o Right'),
	('euro', 'Euro'),
	('eur', 'Eur'),
	('gbp', 'Gbp'),
	('dollar', 'Dollar'),
	('usd', 'Usd'),
	('rupee', 'Rupee'),
	('inr', 'Inr'),
	('cny', 'Cny'),
	('rmb', 'Rmb'),
	('yen', 'Yen'),
	('jpy', 'Jpy'),
	('ruble', 'Ruble'),
	('rouble', 'Rouble'),
	('rub', 'Rub'),
	('won', 'Won'),
	('krw', 'Krw'),
	('bitcoin', 'Bitcoin'),
	('btc', 'Btc'),
	('file', 'File'),
	('file-text', 'File Text'),
	('sort-alpha-asc', 'Sort alpha Asc'),
	('sort-alpha-desc', 'Sort alpha Desc'),
	('sort-amount-asc', 'Sort amount Asc'),
	('sort-amount-desc', 'Sort amount Desc'),
	('sort-numeric-asc', 'Sort numeric Asc'),
	('sort-numeric-desc', 'Sort numeric Desc'),
	('thumbs-up', 'Thumbs Up'),
	('thumbs-down', 'Thumbs Down'),
	('youtube-square', 'Youtube Square'),
	('youtube', 'Youtube'),
	('xing', 'Xing'),
	('xing-square', 'Xing Square'),
	('youtube-play', 'Youtube Play'),
	('dropbox', 'Dropbox'),
	('stack-overflow', 'Stack Overflow'),
	('instagram', 'Instagram'),
	('flickr', 'Flickr'),
	('adn', 'Adn'),
	('bitbucket', 'Bitbucket'),
	('bitbucket-square', 'Bitbucket Square'),
	('tumblr', 'Tumblr'),
	('tumblr-square', 'Tumblr Square'),
	('long-arrow-down', 'Long arrow Down'),
	('long-arrow-up', 'Long arrow Up'),
	('long-arrow-left', 'Long arrow Left'),
	('long-arrow-right', 'Long arrow Right'),
	('apple', 'Apple'),
	('windows', 'Windows'),
	('android', 'Android'),
	('linux', 'Linux'),
	('dribbble', 'Dribbble'),
	('skype', 'Skype'),
	('foursquare', 'Foursquare'),
	('trello', 'Trello'),
	('female', 'Female'),
	('male', 'Male'),
	('gittip', 'Gittip'),
	('gratipay', 'Gratipay'),
	('sun-o', 'Sun o'),
	('moon-o', 'Moon o'),
	('archive', 'Archive'),
	('bug', 'Bug'),
	('vk', 'Vk'),
	('weibo', 'Weibo'),
	('renren', 'Renren'),
	('pagelines', 'Pagelines'),
	('stack-exchange', 'Stack Exchange'),
	('arrow-circle-o-right', 'Arrow circle-o Right'),
	('arrow-circle-o-left', 'Arrow circle-o Left'),
	('toggle-left', 'Toggle Left'),
	('caret-square-o-left', 'Caret square-o Left'),
	('dot-circle-o', 'Dot circle o'),
	('wheelchair', 'Wheelchair'),
	('vimeo-square', 'Vimeo Square'),
	('turkish-lira', 'Turkish Lira'),
	('try', 'Try'),
	('plus-square-o', 'Plus square o'),
	('space-shuttle', 'Space Shuttle'),
	('slack', 'Slack'),
	('envelope-square', 'Envelope Square'),
	('wordpress', 'Wordpress'),
	('openid', 'Openid'),
	('institution', 'Institution'),
	('bank', 'Bank'),
	('university', 'University'),
	('mortar-board', 'Mortar Board'),
	('graduation-cap', 'Graduation Cap'),
	('yahoo', 'Yahoo'),
	('google', 'Google'),
	('reddit', 'Reddit'),
	('reddit-square', 'Reddit Square'),
	('stumbleupon-circle', 'Stumbleupon Circle'),
	('stumbleupon', 'Stumbleupon'),
	('delicious', 'Delicious'),
	('digg', 'Digg'),
	('pied-piper', 'Pied Piper'),
	('pied-piper-alt', 'Pied piper Alt'),
	('drupal', 'Drupal'),
	('joomla', 'Joomla'),
	('language', 'Language'),
	('fax', 'Fax'),
	('building', 'Building'),
	('child', 'Child'),
	('paw', 'Paw'),
	('spoon', 'Spoon'),
	('cube', 'Cube'),
	('cubes', 'Cubes'),
	('behance', 'Behance'),
	('behance-square', 'Behance Square'),
	('steam', 'Steam'),
	('steam-square', 'Steam Square'),
	('recycle', 'Recycle'),
	('automobile', 'Automobile'),
	('car', 'Car'),
	('cab', 'Cab'),
	('taxi', 'Taxi'),
	('tree', 'Tree'),
	('spotify', 'Spotify'),
	('deviantart', 'Deviantart'),
	('soundcloud', 'Soundcloud'),
	('database', 'Database'),
	('file-pdf-o', 'File pdf o'),
	('file-word-o', 'File word o'),
	('file-excel-o', 'File excel o'),
	('file-powerpoint-o', 'File powerpoint o'),
	('file-photo-o', 'File photo o'),
	('file-picture-o', 'File picture o'),
	('file-image-o', 'File image o'),
	('file-zip-o', 'File zip o'),
	('file-archive-o', 'File archive o'),
	('file-sound-o', 'File sound o'),
	('file-audio-o', 'File audio o'),
	('file-movie-o', 'File movie o'),
	('file-video-o', 'File video o'),
	('file-code-o', 'File code o'),
	('vine', 'Vine'),
	('codepen', 'Codepen'),
	('jsfiddle', 'Jsfiddle'),
	('life-bouy', 'Life Bouy'),
	('life-buoy', 'Life Buoy'),
	('life-saver', 'Life Saver'),
	('support', 'Support'),
	('life-ring', 'Life Ring'),
	('circle-o-notch', 'Circle o Notch'),
	('ra', 'Ra'),
	('rebel', 'Rebel'),
	('ge', 'Ge'),
	('empire', 'Empire'),
	('git-square', 'Git Square'),
	('git', 'Git'),
	('hacker-news', 'Hacker News'),
	('tencent-weibo', 'Tencent Weibo'),
	('qq', 'Qq'),
	('wechat', 'Wechat'),
	('weixin', 'Weixin'),
	('send', 'Send'),
	('paper-plane', 'Paper Plane'),
	('send-o', 'Send o'),
	('paper-plane-o', 'Paper plane o'),
	('history', 'History'),
	('genderless', 'Genderless'),
	('circle-thin', 'Circle Thin'),
	('header', 'Header'),
	('paragraph', 'Paragraph'),
	('sliders', 'Sliders'),
	('share-alt', 'Share Alt'),
	('share-alt-square', 'Share alt Square'),
	('bomb', 'Bomb'),
	('soccer-ball-o', 'Soccer ball o'),
	('futbol-o', 'Futbol o'),
	('tty', 'Tty'),
	('binoculars', 'Binoculars'),
	('plug', 'Plug'),
	('slideshare', 'Slideshare'),
	('twitch', 'Twitch'),
	('yelp', 'Yelp'),
	('newspaper-o', 'Newspaper o'),
	('wifi', 'Wifi'),
	('calculator', 'Calculator'),
	('paypal', 'Paypal'),
	('google-wallet', 'Google Wallet'),
	('cc-visa', 'Cc Visa'),
	('cc-mastercard', 'Cc Mastercard'),
	('cc-discover', 'Cc Discover'),
	('cc-amex', 'Cc Amex'),
	('cc-paypal', 'Cc Paypal'),
	('cc-stripe', 'Cc Stripe'),
	('bell-slash', 'Bell Slash'),
	('bell-slash-o', 'Bell slash o'),
	('trash', 'Trash'),
	('copyright', 'Copyright'),
	('at', 'At'),
	('eyedropper', 'Eyedropper'),
	('paint-brush', 'Paint Brush'),
	('birthday-cake', 'Birthday Cake'),
	('area-chart', 'Area Chart'),
	('pie-chart', 'Pie Chart'),
	('line-chart', 'Line Chart'),
	('lastfm', 'Lastfm'),
	('lastfm-square', 'Lastfm Square'),
	('toggle-off', 'Toggle Off'),
	('toggle-on', 'Toggle On'),
	('bicycle', 'Bicycle'),
	('bus', 'Bus'),
	('ioxhost', 'Ioxhost'),
	('angellist', 'Angellist'),
	('cc', 'Cc'),
	('shekel', 'Shekel'),
	('sheqel', 'Sheqel'),
	('ils', 'Ils'),
	('meanpath', 'Meanpath'),
	('buysellads', 'Buysellads'),
	('connectdevelop', 'Connectdevelop'),
	('dashcube', 'Dashcube'),
	('forumbee', 'Forumbee'),
	('leanpub', 'Leanpub'),
	('sellsy', 'Sellsy'),
	('shirtsinbulk', 'Shirtsinbulk'),
	('simplybuilt', 'Simplybuilt'),
	('skyatlas', 'Skyatlas'),
	('cart-plus', 'Cart Plus'),
	('cart-arrow-down', 'Cart arrow Down'),
	('diamond', 'Diamond'),
	('ship', 'Ship'),
	('user-secret', 'User Secret'),
	('motorcycle', 'Motorcycle'),
	('street-view', 'Street View'),
	('heartbeat', '\U$1\L$2'),
	('venus', 'Venus'),
	('mars', 'Mars'),
	('mercury', 'Mercury'),
	('transgender', 'Transgender'),
	('transgender-alt', 'Transgender Alt'),
	('venus-double', 'Venus Double'),
	('mars-double', 'Mars Double'),
	('venus-mars', 'Venus Mars'),
	('mars-stroke', 'Mars Stroke'),
	('mars-stroke-v', 'Mars stroke v'),
	('mars-stroke-h', 'Mars stroke h'),
	('neuter', 'Neuter'),
	('facebook-official', 'Facebook Official'),
	('pinterest-p', 'Pinterest p'),
	('whatsapp', 'Whatsapp'),
	('server', 'Server'),
	('user-plus', 'User Plus'),
	('user-times', 'User Times'),
	('hotel', 'Hotel'),
	('bed', 'Bed'),
	('viacoin', 'Viacoin'),
	('train', 'Train'),
	('subway', 'Subway'),
	('medium','Medium')
)


import uuid
import traceback
import sys

from django.db import models
from django.db.models.query import QuerySet
from django.conf import settings
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _
from django.utils.text import slugify
from django.utils.timezone import now
from django.utils.safestring import mark_safe
from django.utils.functional import cached_property
from django.utils.safestring import mark_safe
try:
    from django.apps import apps
    get_model = apps.get_model
except:
    from django.db.models.loading import get_model



from carbon.utils.slugify import unique_slugify
from carbon.utils.template import get_template_by_pk_or_slug

# -- Level 1
class VersionableAtom(models.Model):


    version = models.IntegerField(default=0) #THOUGHT: Is this useful??
    created_date = models.DateTimeField(_('Created Date'), auto_now_add=True, 
        blank=True, null=True)
    created_by = models.ForeignKey(settings.AUTH_USER_MODEL, 
        blank=True, null=True, related_name='%(app_label)s_%(class)s_created_by',
        on_delete=models.SET_NULL)

    modified_date = models.DateTimeField(_('Modified Date'), auto_now=True, 
        blank=True, null=True)
    modified_by = models.ForeignKey(settings.AUTH_USER_MODEL, 
        blank=True, null=True, related_name='%(app_label)s_%(class)s_modified_by',
        on_delete=models.SET_NULL)

    admin_note = models.TextField(_('admin note'), blank=True, null=True,
        help_text="Not publicly visible")

    class Meta:
        abstract = True

    # def __init__(self, *args, **kwargs):
    #     super(Versionable, self).save(*args, **kwargs)

    @property
    def edit_item_url(self):
        if self.pk:
            object_type = type(self).__name__
            url = reverse('admin:%s_%s_change' %(self._meta.app_label,  self._meta.model_name),  args=[self.id] )
            return url
        return None

    @property
    def django_id(self):
        return u"%s.%s.%s"%(self._meta.app_label, self._meta.object_name, self.pk)

    def edit_item(self):
        style="style='width:278px;display:block;'"
        if self.pk:            
            try:
                url = self.edit_item_url
                return mark_safe('<a href="%s" %s>Edit Item &gt;</a>'%(url, style))
            except:
                return mark_safe('<span %s>&nbsp;</span>'%(style))
        return mark_safe('<span %s>&nbsp;</span>'%(style))
    

    @property
    def is_edited(self):
        return self.created_date != self.modified_date

    @cached_property
    def abstract_children(self):
        try:
            children = super(VersionableAtom, self).get_children()
        except:
            children = []
        return children


    def get_children(self):
        return self.abstract_children


    @cached_property
    def siblings(self):
        try:
            siblings = super(VersionableAtom, self).get_siblings()
        except:
            siblings = self.__class__.objects.all()
        return siblings

    def get_siblings(self):
        self.siblings

    def is_published(self):
        try:
            published = super(VersionableAtom, self).is_published()
        except:
            published = True
        return published


    def save(self, *args, **kwargs):
        
        self.increment_version_number()

        super(VersionableAtom, self).save(*args, **kwargs)

    def increment_version_number(self):
        self.version = self.version+1

# -- Level 1a
class HierarchicalAtom(models.Model):


    parent = models.ForeignKey('self', blank=True, null=True,
        on_delete=models.SET_NULL)

    @cached_property
    def hierarchical_children(self):
        all_children = self.__class__.objects.filter(parent=self).order_by('order')
        return [child for child in all_children]

    def get_children(self, require_published=True):
        all_children = self.hierarchical_children
        if require_published:
            return [child for child in all_children if child.is_published()]
        return all_children

    @cached_property
    def published_siblings(self):
        if self.parent:
            return self.parent.get_children(True)
        siblings = self.__class__.objects.all()
        return [sibling for sibling in siblings if sibling.is_published()]
        

    @cached_property
    def siblings(self):
        if self.parent:
            return self.parent.get_children()
        return self.__class__.objects.all()

    def get_siblings(self, require_published=True):
        if require_published:
            return self.published_siblings
        else:
            return self.siblings
        

    def get_next_sibling(self, siblings=None, require_published=True):
        if siblings == None:
            siblings = self.get_siblings(require_published)

        if len(siblings)==0:
            return None

        if isinstance(siblings, QuerySet):
            ids = list(siblings.values_list('id', flat=True))
        else:
            ids = [sibling.id for sibling in siblings]

        try:
            index = ids.index(self.id)
        except:
            index = None

        next_item = None
        next_index = None
        if index != None:
            next_index = (index + 1) if index < len(siblings)-1 else None
            
        if next_index != None:
            next_item = siblings[next_index]
        

        return next_item

    def get_previous_sibling(self, siblings=None, require_published=True):
        if siblings == None:
            siblings = self.get_siblings(require_published)

        if len(siblings)==0:
            return None

        if isinstance(siblings, QuerySet):
            ids = list(siblings.values_list('id', flat=True))
        else:
            ids = [sibling.id for sibling in siblings]

        try:
            index = ids.index(self.id)
        except:
            index = None

        previous_item = None
        previous_index = None
        if index != None:
            previous_index = (index -1 ) if index > 0 else None
            
        if previous_index != None:
            previous_item = siblings[previous_index]
        
        return previous_item

    @property
    def edit_parent_url(self):
        if self.parent:
            object_type = type(self.parent).__name__
            return reverse('admin:%s_%s_change' %(self.parent._meta.app_label,  self.parent._meta.model_name),  args=[self.parent.id] )
        return None

    def edit_parent(self):
        style="style='width:278px;display:block;'"
        if self.parent:
            
            try:
                url = self.edit_parent_url
                return mark_safe('<a href="%s" %s>&lt; Edit Parent</a>'%(url, style))
            except:
                return mark_safe('<span %s>&nbsp;</span>'%(style))
        return mark_safe('<span %s>&nbsp;</span>'%(style))
    

    def get_hierarchy(self):
        if self.parent:
            return self.parent.get_hierarchy() + [self]
        return [self]

    def get_next_child(self, item):
        children = self.get_children()
        next_index = (children.index(item) + 1) % len(children)
        return children[next_index]

    def get_previous_child(self, item):
        children = self.get_children()
        previous_index = (children.index(item) + len(children) - 1) % len(children)
        return children[previous_index]


    class Meta:
        abstract = True

# # -- Level 1b
# from django.contrib.contenttypes import generic
# from django.contrib.contenttypes.models import ContentType
# class Flat(models.Model):

#     def get_parent_model():
#         raise NotImplementedError('Class should specify parent model')

#     content_type = models.ForeignKey(ContentType, limit_choices_to={"model__in": self.get_parent_model()}))
#     object_id = models.PositiveIntegerField()
#     content_object = GenericForeignKey('content_type', 'object_id')

#     class Meta:
#         abstract = True

#     @property
#     def parent(self):
#         return self.content_object

#     @staticmethod
#     def get_by_parent(search_class, parent):
#         return search_class.objects.filter(object_id=parent.id, content_type=ContentType.objects.get_for_model(parent))


# -- Level 1.5
class TitleAtom(models.Model):

    help = {
        'title': "The display title for this object.",
        'slug': "Auto-generated page slug for this object.",
    }

    title = models.CharField(_('Title'), max_length=255, 
        help_text=help['title'], blank=True, null=True)

    slug = models.CharField(_('Slug'), max_length=255, blank=True, 
        db_index=True, help_text=help['slug'])

    class Meta:
        abstract = True
        ordering = ['title']

    def __unicode__(self):
        return self.title

    @staticmethod
    def autocomplete_search_fields():
        return ("title__icontains",)

    def generate_slug(self):
        unique_slugify(self, self.title)
        return self.slug

    def generate_title(self):
        return 'Untitled %s'%(self.__class__.__name__)

    def verify_title_and_slug(self):
        if not self.title:
            self.title = self.generate_title()

        if not self.slug or not self.pk:
            self.slug = self.generate_slug()
        
    def save(self, *args, **kwargs):

        self.verify_title_and_slug()

        super(TitleAtom, self).save(*args, **kwargs)


class OrderedItemAtom(models.Model):

    help = {
        'order': "",
    }

    #When implementing, specify item and tag FKs:
    #tag = models.ForeignKey('app.Model')
    #item = models.ForeignKey('app.Model')
    order = models.IntegerField(default=0, help_text=help['order'])

    class Meta:
        abstract = True  
        ordering = ['order']


# -- Level 2
class AddressibleAtom(TitleAtom):
   
    help = {
        'uuid': "UUID generated for object; can be used for short URLs",
        'parent': "Hierarchical parent of this item. Used to define path.",
        'path': 'Actual path used based on generated and override path',
        'path_generated': "The URL path to this page, based on page hierarchy and slug.",
        'path_override': "The URL path to this page, defined absolutely.",
        'hierarchy': "Administrative Hierarchy",
        'temporary_redirect': "Temporarily redirect to a different path",
        'permanent_redirect': "Permanently redirect to a different path",
        'template': 'Template for view',
        'order': "",
    }
    

    uuid = models.CharField(_('UUID'), max_length=255, blank=True, 
        help_text=help['uuid'])


    order = models.IntegerField(default=0, help_text=help['order']) 
    

    template = models.ForeignKey(settings.TEMPLATE_MODEL, null=True, blank=True,
        help_text=help['template'])    

    path = models.CharField(_('path'), max_length=255, 
        help_text=help['path'], blank=True, null=True)
    title_path = models.CharField(_('title path'), max_length=255, 
        help_text=help['path'], blank=True, null=True)
    path_generated = models.CharField(_('generated path'), max_length=255, 
        help_text=help['path_generated'], blank=True, null=True)
    path_override = models.CharField(_('path override'), max_length=255,
        help_text=help['path_override'], blank=True, null=True)
    hierarchy = models.CharField(_('hierarchy'), max_length=255, unique=True,
        null=True, blank=True, help_text=help['hierarchy'])

    temporary_redirect = models.CharField(_('Temporary Redirect'), max_length=255,
        blank=True, help_text=help['temporary_redirect'])
    permanent_redirect = models.CharField(_('Permanent Redirect'), max_length=255,
        blank=True, help_text=help['permanent_redirect'])


    class Meta:
        abstract = True
        ordering = ['hierarchy']

    def __unicode__(self):
        return self.title

    @staticmethod
    def autocomplete_search_fields():
        return ("title__icontains",)

    @staticmethod
    def get_by_uuid(uuid):
        if uuid:
            split = uuid.split('.')
            try:
                app_label = split[0]
                object_name = split[1]
                uuid = split[2]

                model = get_model(app_label, object_name)
                instance = model.objects.get(uuid=uuid)
                return instance
            except Exception, err:
                if settings.DEBUG:
                    print 'Error getting object by uuid %s - %s'%(traceback.format_exc(), sys.exc_info()[0])
        return None

    @property
    def is_external(self):
        if self.path_override:
            if self.path_override.startswith('/'):
                return False
            return True
        return False

    def get_url_path(self):
        path = self.path
        if path.startswith('/'):
            path = path[1:]

        if path.endswith('/'):
            path = path[:-1]

        return path

    def build_path(self, path_attribute='slug', parent_path_attribute='path'):

        try: 
            model = type(self)
            model._meta.get_field('parent')
            has_parent = True 
        except models.FieldDoesNotExist:
            has_parent = False

        if has_parent and self.parent:
            parent_path = getattr(self.parent, parent_path_attribute)
            if parent_path and not parent_path.endswith('/'):
                parent_path = "%s/"%(parent_path)

            
            return "%s%s/" % (parent_path, getattr(self, path_attribute))
        else:
            return "/%s/" % getattr(self, path_attribute)

    def build_hierarchy_path(self):

        try: 
            model = type(self)
            model._meta.get_field('parent')
            has_parent = True 
        except models.FieldDoesNotExist:
            has_parent = False

        if has_parent and self.parent:
            return "%s%s/%s/" % (self.parent.build_hierarchy_path(), str(self.order).zfill(4), self.slug)
        else:
            return "/%s/%s/" % (str(self.order).zfill(4), self.slug)

    @property
    def admin_hierarchy_display(self):

        try: 
            model = type(self)
            model._meta.get_field('parent')
            has_parent = True 
        except models.FieldDoesNotExist:
            has_parent = False

        if has_parent and self.parent:
            return "%s<span style='color:#fff'>|------</span>" % self.parent.admin_hierarchy_display
        else:
            return ''

    @property
    def admin_hierarchy(self):

        try: 
            model = type(self)
            model._meta.get_field('parent')
            has_parent = True 
        except models.FieldDoesNotExist:
            has_parent = False

        if has_parent and self.parent:
            return mark_safe("%s&lfloor; %s" % (self.admin_hierarchy_display, self.title))
        else:
            return self.title


    def generate_slug(self):
        unique_slugify(self, self.title)
        return self.slug

    def generate_path(self):
        if self.path_override != None and self.path_override != '':
            return  self.path_override
        return self.path_generated

    def get_absolute_url(self):
        return self.path

    def get_default_template(self):

        found_template = None

        if hasattr(self, 'default_template') and self.default_template:
            
            found_template = get_template_by_pk_or_slug(self.default_template)

        return found_template

    @property
    def edit_template_url(self):
        if self.template:
            object_type = type(self.template).__name__
            return reverse('admin:%s_%s_change' %(self.template._meta.app_label,  self.template._meta.model_name),  args=[self.template.id] )
        return None

    def edit_template(self):
        style="style='width:278px;display:block;'"
        if self.template:
            
            try:
                url = self.edit_template_url
                return mark_safe('<a href="%s" %s>Edit Template &gt;</a>'%(url, style))
            except:
                return mark_safe('<span %s>&nbsp;</span>'%(style))
        return mark_safe('<span %s>&nbsp;</span>'%(style))
    

    def save(self, *args, **kwargs):
        if not self.template:
            use_default_template = self.get_default_template()
            if use_default_template:
                self.template = use_default_template
        
        try: 
            model = type(self)
            model._meta.get_field('parent')
            has_parent = True 
        except models.FieldDoesNotExist:
            has_parent = False

        #Dont let parent point to self
        if has_parent and self.parent:
            if self.parent == self:
                self.parent = None

        if not self.uuid:
            app_label = type(self)._meta.app_label
            object_name = type(self)._meta.object_name
            self.uuid = "%s.%s.%s"%(app_label, object_name, uuid.uuid1())

        original = None
        if self.pk is not None:
            model = type(self)
            original = model.objects.get(pk=self.pk)
        
        
        self.verify_title_and_slug()

        self.path_generated = self.build_path('slug', 'path')

        self.title_path = self.build_path('title', 'title_path')

        self.path = self.generate_path()

        

        self.hierarchy = self.build_hierarchy_path()

        path_has_changed = False
        if original != None:
            if (original.path != self.path):
                path_has_changed = True


        super(AddressibleAtom, self).save(*args, **kwargs)

        #If path has changed, notify children
        if path_has_changed:
            [p.save() for p in self.get_children()]






from django import forms
from django.conf import settings

from ckeditorfiles.widgets import CKEditorWidget, CKEditorInlineWidget
# from .models import Page, PageContentBlock

class PageAdminForm(forms.ModelForm):
	content = forms.CharField(widget=CKEditorWidget(config=settings.CKEDITOR_CONFIGS['page_content_ckeditor']), required=False)
	synopsis = forms.CharField(widget=CKEditorWidget(config=settings.CKEDITOR_CONFIGS['page_synopsis_ckeditor']), required=False)
	class Meta:
		# model = Page
		fields = '__all__'

class GlobalContentBlockAdminForm(forms.ModelForm):
	content = forms.CharField(widget=CKEditorWidget(config=settings.CKEDITOR_CONFIGS['page_content_ckeditor']), required=False)
	class Meta:
		# model = GlobalContentBlock
		fields = '__all__'		


class PageContentBlockAdminForm(forms.ModelForm):
	content = forms.CharField(widget=CKEditorInlineWidget(config=settings.CKEDITOR_CONFIGS['pagecontentblock_content_ckeditor']), required=False)
	synopsis = forms.CharField(widget=CKEditorInlineWidget(config=settings.CKEDITOR_CONFIGS['pagecontentblock_synopsis_ckeditor']), required=False)
	
	class Meta:
		# model = PageContentBlock
		fields = '__all__'




from django.db import models
from django.conf import settings
from django.core.urlresolvers import reverse
from django.template import Template as DjangoTemplate
from django.template import loader
from django.template.loaders.filesystem import Loader as FileSystemLoader
from django.utils.text import Truncator
from django.utils.timezone import now
from django.utils.translation import ugettext_lazy as _
from django.utils.functional import cached_property
from django.utils.safestring import mark_safe

from carbon.utils.slugify import unique_slugify
from carbon.utils.template import get_page_templates, get_page_templates_raw

from carbon.atoms.managers.content import PublishableManager

from .abstract import *


class PublishableAtom(models.Model):
    publish_by_default = False

    help = {
        'publication_status': "Current publication status",
        'publish_on_date': "Object state will be set to 'Published' on this date.",
        'expire_on_date': "Object state will be set to 'Expired' on this date.",
    }

    DRAFT = 10
    REVIEW = 20
    PUBLISHED = 100
    #EXPIRED = 30
    UNPUBLISHED = 40
    PUB_STATUS_CHOICES = (
        (DRAFT, _("Draft")),
        (REVIEW, _("Needs Review")),
        (PUBLISHED, _("Published")),
        # (EXPIRED, _("Expired")),
        (UNPUBLISHED, _("Unpublished")),
    )

    objects = PublishableManager()

    publication_date = models.DateTimeField(_('Publication Date'), blank=True, null=True)
    published_by = models.ForeignKey(settings.AUTH_USER_MODEL, 
        blank=True, null=True, related_name='%(app_label)s_%(class)s_published_by', 
        on_delete=models.SET_NULL)

    publication_status = models.IntegerField(choices=PUB_STATUS_CHOICES, 
        default=DRAFT, help_text=help['publication_status'])
    

    publish_on_date = models.DateTimeField(_('Publish on Date'), 
        blank=True, null=True, help_text=help['publish_on_date'])
    expire_on_date = models.DateTimeField(_('Expire on Date'), 
        blank=True, null=True, help_text=help['expire_on_date'])

    def is_published(self):
        
        if self.publication_status == PublishableAtom.PUBLISHED:

            if self.publish_on_date and now() < self.publish_on_date:
                return False

            if self.expire_on_date and now() > self.expire_on_date:
                return False

            return True

        return False

    def save(self, *args, **kwargs):

        #Published by default
        if not self.pk and self.publish_by_default:
            self.publication_status = PublishableAtom.PUBLISHED

        has_been_published = False
        if self.pk is not None and self.publication_status == PublishableAtom.PUBLISHED:
            original = type(self).objects.get(pk=self.pk)
            if original.publication_status != self.publication_status:
                has_been_published = True

        #If just published, set published date
        if has_been_published and not self.publication_date:
            self.publication_date = now()

        super(PublishableAtom, self).save(*args, **kwargs)

    class Meta:
        abstract = True


class SEOAtom(models.Model):
   
    help = {
        'page_meta_description': "A short description of the page, used for SEO and not displayed to the user; aim for 150-160 characters.",
        'page_meta_keywords': "A short list of keywords of the page, used for SEO and not displayed to the user; aim for 150-160 characters.",
        'is_searchable': "Allow search engines to index this object and display in sitemap.",
        'in_sitemap':"Is in sitemap",
        'noindex':"Robots noindex",
        'nofollow':"Robots nofollow",
        'sitemap_changefreq': "How frequently does page content update",
        'sitemap_priority': "Sitemap priority",
    }

    NEVER = 'never'
    YEARLY = 'yearly'
    MONTHLY = 'monthly'
    WEEKLY = 'weekly'
    DAILY = 'daily'
    HOURLY = 'hourly'
    ALWAYS = 'always'
    CHANGE_FREQ_CHOICES = (
        (NEVER, _("Never")),
        (YEARLY, _("Yearly")),
        (MONTHLY, _("Monthly")),
        (WEEKLY, _("Weekly")),
        (DAILY, _("Daily")),
        (HOURLY, _("Hourly")),
        (ALWAYS, _("Always")),
    )

    #SEO
    page_meta_description = models.CharField(_('Meta Description'), 
        max_length=2000, blank=True, help_text=help['page_meta_description'])
    page_meta_keywords = models.CharField(_('Meta Page Keywords'), 
        max_length=2000, blank=True, help_text=help['page_meta_keywords'])

    is_searchable = models.BooleanField(default=True, help_text=help['is_searchable'])
    in_sitemap = models.BooleanField(default=True, help_text=help['in_sitemap'])
    noindex = models.BooleanField(default=False, help_text=help['noindex'])
    nofollow = models.BooleanField(default=False, help_text=help['nofollow'])
    sitemap_changefreq = models.CharField(_('Sitemap Change Frequency'), max_length=255, 
        default=MONTHLY, choices=CHANGE_FREQ_CHOICES,help_text=help['sitemap_changefreq'])
    sitemap_priority = models.CharField("Sitemap Priority", max_length=255, 
        null=True, blank=True, default='0.5', help_text=help['sitemap_priority'])

    class Meta:
        abstract = True

    #Sitemap Properties
    def lastmod(self):
        if hasattr(self, 'modified'):
            return getattr(self, 'modified')
        return None

    def location(self, obj):
        try:
            return obj.get_absolute_url()
        except:
            return None

    def changefreq(self, obj):
        return self.changefreq

class SocialSharingAtom(models.Model):  

    help = {
        'shareable': "Show sharing widget",
        'tiny_url': "Tiny URL used for social sharing",
        'social_share_image': "Standards for the social share image vary, but an image at least 300x200px should work well.",
        'facebook_author_id': "Numeric Facebook ID",
        'twitter_author_id': "Twitter handle, including \"@\" e.g. @cgpartners",
        'google_author_id': "Google author id, e.g. the AUTHOR_ID in https://plus.google.com/AUTHOR_ID/posts",
    }

    # -- Choice Data
    MUSIC_SONG = 'music.song'
    MUSIC_ALBUM = 'music.album'
    MUSIC_PLAYLIST = 'music.playlist'
    MUSIC_PLAYLIST = 'music.radio_station'
    VIDEO_MOVIE = 'video.movie'
    VIDEO_EPISODE = 'video.episode'
    VIDEO_TV_SHOW = 'video.tv_show'
    VIDEO_OTHER = 'video.other'
    ARTICLE = 'article'
    BOOK = 'book'
    PROFILE = 'profile'
    WEBSITE = 'website' 

    TYPE_CHOICES = (
        (ARTICLE, "Article"),
        (BOOK, "Book"),
        (PROFILE, "Profile"),
        (WEBSITE, "Website"),
        (VIDEO_MOVIE, "Video - Movie"),
        (VIDEO_EPISODE, "Video - Episode"),
        (VIDEO_TV_SHOW, "Video - TV Show"),
        (VIDEO_OTHER, "Video - Other"),
        (MUSIC_SONG, "Music - Song"),
        (MUSIC_ALBUM, "Music - Album"),
        (MUSIC_PLAYLIST, "Music - Playlist"),
        (MUSIC_PLAYLIST, "Music - Radio Station"),
    )

    #SOCIAL
    shareable = models.BooleanField(default=False, 
        help_text=help['shareable'])
    tiny_url = models.CharField(_('tiny url'), max_length=255, 
        help_text=help['tiny_url'], null=True, blank=True)

    social_share_type = models.CharField("Social type", max_length=255, 
        null=True, blank=True, choices=TYPE_CHOICES, default=ARTICLE )
    
    social_share_image = models.ForeignKey(settings.IMAGE_MODEL, 
        blank=True, null=True, related_name='%(app_label)s_%(class)s_social_images',
        help_text=help['social_share_image'], on_delete=models.SET_NULL)

    facebook_author_id = models.CharField("Facebook Author ID", max_length=255, 
        null=True, blank=True, help_text=help['facebook_author_id'])
    twitter_author_id = models.CharField("Twitter Admin ID", max_length=255, 
        null=True, blank=True, help_text=help['twitter_author_id'])
    google_author_id = models.CharField("Google Admin ID", max_length=255, 
        null=True, blank=True, help_text=help['google_author_id'])

    

    class Meta:
        abstract = True

    def get_sharing_url(self, service):
        try:
            return self.get_absolute_url()     
        except:
            return None


class ContentAtom(models.Model):
    auto_synopsis_length = 30
    help = {
        'content': "",
        'synopsis': ""
    }

    content = models.TextField(_('content'), help_text=help['content'], null=True, blank=True)
    synopsis = models.TextField(_('synopsis'), help_text=help['synopsis'], null=True, blank=True)

    def get_synopsis(self):
        if self.synopsis and self.synopsis != '':
            return self.synopsis
        elif self.content and self.content != '':
            return Truncator(self.content).words(self.auto_synopsis_length, html=True)
        return ''            

    class Meta:
        abstract = True

class HasImageAtom(models.Model):
    help = {
        'image': "Featured image",
    }
    image = models.ForeignKey(settings.IMAGE_MODEL, 
        blank=True, null=True, related_name='%(app_label)s_%(class)s_images',
        help_text=help['image'], on_delete=models.SET_NULL)

    @cached_property
    def image_preview_url(self):
        if self.image:
            return self.image.thumbnail.url
        return None

    def image_preview(self):
        if self.image:
            try:
                url = self.image_preview_url
                return mark_safe("<img src='%s' alt='%s' />"%(url, self.image.alt))
            except:
                return "Error displaying image"
        else:
            return "-- No image --"     

    class Meta:
        abstract = True        


class LinkAtom(models.Model):

    help = {
        'target': "",
        'css_classes':"Extra css classes to add to the item",
        'extra_attributes': "Extra attributes to add to the item"
    }

    

    BLANK = '_blank'
    SELF = '_self'
    PARENT = '_parent'
    TOP = '_top'
    TARGET_CHOICES = (
        (BLANK, _(BLANK)),
        (SELF, _(SELF)),
        (PARENT, _(PARENT)),
        (TOP, _(TOP))        
    )

    url = models.CharField(_("URL"), max_length=255, blank = True, 
        null = True)

    target = models.CharField(_('Target'), max_length=255, 
        help_text=help['target'], choices=TARGET_CHOICES, default=SELF)

    css_classes = models.CharField(_('CSS Classes'), max_length=255, 
        help_text=help['css_classes'], null=True, blank=True, default='')

    extra_attributes = models.CharField(_('Extra Attributes'), max_length=255, 
        help_text=help['extra_attributes'], null=True, blank=True, default='')

    def get_target(self):
        return 'target="%s"'%(self.target)


    class Meta:
        abstract = True    

class ModerationAtom(models.Model):

    help = {
        'moderation_status': "",
        'user':"",
        'content_cleaned':"Cleaned content",
        'moderation_comment':"Comment from moderator"
    }

    SUBMITTED = 10
    REJECTED = 50
    PUBLISHED = 100    
    APPROVED = 150
    MODERATION_STATUS_CHOICES = (
        (SUBMITTED, _("Submitted by User")),
        (PUBLISHED, _("Published")),
        (REJECTED, _("Rejected by Moderator")),
        (APPROVED, _("Approved by Moderator")),
    )

    moderation_status = models.IntegerField(choices=MODERATION_STATUS_CHOICES, 
        default=SUBMITTED, help_text=help['moderation_status'])

    moderation_comment = models.TextField(_('Moderation Comment'), blank=True,
        help_text=help['moderation_comment'])

    user = models.ForeignKey(settings.AUTH_USER_MODEL, 
        blank=True, null=True, related_name='%(app_label)s_%(class)s_user',
        help_text=help['user'])
    
    cleaned_content = models.TextField(_('Cleaned Content'), blank=True,
        help_text=help['content_cleaned'])


    def is_moderated(self):

        if self.moderation_status == ModerationAtom.PUBLISHED:

            return True
            
        return False

    class Meta:
        abstract = True





class ContentMolecule(VersionableAtom, AddressibleAtom, PublishableAtom, SEOAtom, SocialSharingAtom, ContentAtom, HasImageAtom):
    class Meta:
        abstract = True

    @staticmethod
    def autocomplete_search_fields():
        return ("admin_note__icontains","title__icontains")

class CategoryMolecule(HierarchicalAtom, ContentMolecule):
    #Basically just a tag but with hierarchy
    item_class = None
    item_classes = None
    category_property_name = 'category'

    class Meta:
        abstract = True

    @cached_property
    def category_children(self):
        items = self.get_items()
        output = []
        for child in items:
            if child.is_published():
                if hasattr(self, 'item'):
                    output.append(child.item)
                else:
                    output.append(child)

        return output

    def get_items(self):
        if not self.item_class and not self.item_classes:
            raise NotImplementedError('Class should specify an item_class or item_classes value')
        
        if self.item_classes:
            [item_class.objects.published().filter(**{ self.tag_property_name: self }).order_by('order') for item_class in self.item_classes]
        else:
            return self.item_class.objects.published().filter(**{ self.tag_property_name: self }).order_by('order')

    def get_children(self):
        return self.category_children

    def get_next_item(self, item):
        children = self.get_children()
        next_index = (children.index(item) + 1) % len(children)
        return children[next_index]

    def get_previous_item(self, item):
        children = self.get_children()
        previous_index = (children.index(item) + len(children) - 1) % len(children)
        return children[previous_index]

    @staticmethod
    def autocomplete_search_fields():
        return ("admin_note__icontains","title__icontains")



class TagMolecule(ContentMolecule):
                
    item_class = None
    item_classes = None
    tag_property_name = 'tags'

    class Meta:
        abstract = True    

    @cached_property
    def tag_children(self):  
        if not self.item_class and not self.item_classes:
            raise NotImplementedError('Class should specify an item_class or item_classes value')
        
        if self.item_classes:
            [item_class.objects.published().filter(**{ self.tag_property_name: self }).order_by('order') for item_class in self.item_classes]
        else:
            return self.item_class.objects.published().filter(**{ self.tag_property_name: self }).order_by('order')

    def get_children(self):
        return self.tag_children        

    def get_next_item(self, item):
        children = self.get_children()
        next_index = (children.index(item) + 1) % len(children)
        return children[next_index]

    def get_previous_item(self, item):
        children = self.get_children()
        previous_index = (children.index(item) + len(children) - 1) % len(children)
        return children[previous_index]

    @staticmethod
    def autocomplete_search_fields():
        return ("admin_note__icontains","title__icontains")

class UserInputMolecule(VersionableAtom, AddressibleAtom, PublishableAtom, ContentAtom, ModerationAtom):
    class Meta:
        abstract = True

    @staticmethod
    def autocomplete_search_fields():
        return ("admin_note__icontains","title__icontains")


class UpvoteDownvoteFlagMolecule(VersionableAtom):
    class Meta:
        abstract = True

    help = {
        'type': "",
        'voter':""
    }

    UPVOTE = 'upvote'
    DOWNVOTE = 'downvote'
    FLAG = 'flag'
    VOTE_CHOICES = (
        (UPVOTE, _(UPVOTE)),
        (DOWNVOTE, _(DOWNVOTE)),
        (FLAG, _(FLAG))  
    )

    type = models.CharField(_('Vote Choices'), max_length=255, 
        help_text=help['type'], choices=VOTE_CHOICES)

    voter = models.ForeignKey(settings.AUTH_USER_MODEL, 
        blank=True, null=True, related_name='%(app_label)s_%(class)s_user',
        help_text=help['voter'])

    # Implement what user is voting on:
    # item = models.ForeignKey('blog.BlogComment')

class HasSlidesMolecule(models.Model):
    #TODO: Defile
    #slide_class

    @cached_property
    def slides(self):
        return self.slide_class.objects.filter(parent=self).order_by('order').select_related('slide_image')

    class Meta:
        abstract = True

class SlideMolecule(VersionableAtom, OrderedItemAtom):
    
    # parent = models.ForeignKey('app.Model')
    slide_image = models.ForeignKey('media.Image', null=True, blank=True)

    class Meta:
        ordering = ['order']
        abstract = True

        